/**************************************************************************************

                            Data Manipulation Functions

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "data.h"
#include "table.h"
#include "offsets.h"
#include "allocator.h"
#include "hash.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

//String converter
char* ndb_itoa(uint64_t i) {
	static char buf[21] = {0};
	sprintf(buf,"%lu",i);
	return buf;
}

NDB_SIZE ndb_dtSizeTo(nDB* db, NDB_LOC tbl, uint8_t fieldnum) {
	NDB_SIZE size = 0;
	uint8_t fnum;
	uint8_t i;
	assert(db && tbl);
	
	fnum = ndb_otsNum(db,tbl);	//Gets the number of fields
	
	assert(fnum);				//Make sure there ARE fields
	
	//This is to make sure that the fieldnum it asks for is valid for this table.
	assert(fnum >= fieldnum);
	
	//So the fieldnum is valid, we can start calculating the size:
	
	//For each field up until the number of field that we want, add the
	//	size it takes up
	for (i=0; i < fieldnum;i++) {
		switch (ndb_tfieldType(db,tbl,i)) {
			case NDB_TU8:
				size+= NDB_TSU8;
				break;
			case NDB_TU16:
				size+= NDB_TSU16;
				break;
			case NDB_TU32:
				size+= NDB_TSU32;
				break;
			case NDB_TU64:
				size+= NDB_TSU64;
				break;
			case NDB_TSTR:
				size+= NDB_TSSTR;
				break;
			default:
				ndb_log(NDB_LTBL,"data: Field type unrecognized! Database might be corrupted!\n");
				ndb_errSet(db,15);
				return 0;
		}
	}
	
	//Returns the size calculated
	return size;
}

//Internal statements: Assume correct type

//The internal get statements
uint8_t ndb_dgu8(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU8);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fit within the place! Return it!
		return ndb_pu8(db,NDB_DATALOC(loc)+s);
	} else {
		//The table had to be allocated, so return the allocated unit
		return ndb_pu8(db,ndb_odData(db,loc)+s);
	}
	
}
uint16_t ndb_dgu16(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU16);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fit within the place! Return it!
		return ndb_pu16(db,NDB_DATALOC(loc)+s);
	} else {
		//The table had to be allocated, so return the allocated unit
		return ndb_pu16(db,ndb_odData(db,loc)+s);
	}
	
}
uint32_t ndb_dgu32(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU32);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fit within the place! Return it!
		return ndb_pu32(db,NDB_DATALOC(loc)+s);
	} else {
		//The table had to be allocated, so return the allocated unit
		return ndb_pu32(db,ndb_odData(db,loc)+s);
	}
	
}
uint64_t ndb_dgu64(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU64);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fit within the place! Return it!
		return ndb_pu64(db,NDB_DATALOC(loc)+s);
	} else {
		//The table had to be allocated, so return the allocated unit
		return ndb_pu64(db,ndb_odData(db,loc)+s);
	}
	
}
char* ndb_dgstr(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TSTR);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Now we check if it is allocated or not
	if (ndb_odBoolGet(db,loc,0)) {
		//It was not allocated, so we add the location of the data to the offset
		s += NDB_DATALOC(loc);
	} else {
		//It is allocated, so we add the location of allocation to the offset
		s += ndb_odData(db,loc);
	}
	
	//So s is the location of the string allocation
	if (ndb_ploc(db,s+sizeof(uint16_t))) {
		return ndb_pstr(db,ndb_ploc(db,s+sizeof(uint16_t)));
	}
	return NULL;
	
}

uint16_t ndb_dgshash(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TSTR);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Now we check if it is allocated or not
	if (ndb_odBoolGet(db,loc,0)) {
		//It is allocated, so we add the location of allocation to the offset
		s += ndb_odData(db,loc);
	} else {
		//It was not allocated, so we add the location of the data to the offset
		s += NDB_DATALOC(loc);
	}
	return ndb_pu16(db,s);
}

//-----------------------------------------------------------------------------------
//Internal set statements
void ndb_dsu8(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint8_t res) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU8);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fits within the place - set it!
		ndb_pu8(db,NDB_DATALOC(loc)+s) = res;
	} else {
		//The table had to be allocated, so set the allocated unit
		ndb_pu8(db,ndb_odData(db,loc)+s) = res;
	}
	
}
void ndb_dsu16(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint16_t res) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU16);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fits within the place - set it!
		ndb_pu16(db,NDB_DATALOC(loc)+s) = res;
	} else {
		//The table had to be allocated, so set the allocated unit
		ndb_pu16(db,ndb_odData(db,loc)+s) = res;
	}
	
}
void ndb_dsu32(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint32_t res) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU32);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fits within the place - set it!
		ndb_pu32(db,NDB_DATALOC(loc)+s) = res;
	} else {
		//The table had to be allocated, so set the allocated unit
		ndb_pu32(db,ndb_odData(db,loc)+s) = res;
	}
	
}
void ndb_dsu64(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint64_t res) {
	NDB_LOC s;
	assert(db && loc && tbl);
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TU64);
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Check if it is allocated, or if it is in-place
	if (ndb_odBoolGet(db,loc,0)) {
		//It fits within the place - set it!
		ndb_pu64(db,NDB_DATALOC(loc)+s) = res;
	} else {
		//The table had to be allocated, so set the allocated unit
		ndb_pu64(db,ndb_odData(db,loc)+s) = res;
	}
	
}
void ndb_dsstr(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, char* res) {
	NDB_LOC s,size;
	
	assert(db && loc && tbl && res);	//Make sure the arguments exist	
	
	//It must be the same type
	assert(ndb_tfieldType(db,tbl,field) == NDB_TSTR);
	
	
	//Get the size TO the field
	s = ndb_dtSizeTo(db,tbl,field);
	
	//Now we check if it is allocated or not
	if (ndb_odBoolGet(db,loc,0)) {
		//It was not allocated, so we add the location of the data to the offset
		s += NDB_DATALOC(loc);
	} else {
		//It is allocated, so we add the location of allocation to the offset
		s += ndb_odData(db,loc);
	}
	
	//So s is the location of the string allocation
	
	//We now get the size of the new string
	size = strlen(res);
	
	//Next is the allocation-check - making sure that such an allocation exists
	if (ndb_ploc(db,s+sizeof(uint16_t))) {
		//There exists a previous allocation. Now there are two options - the previous
		//	allocation is large enough to hold the new string, or it is not large enough.
		
		//Obtain the length of the old string
		NDB_LOC sizeold = strlen(ndb_pstr(db,ndb_ploc(db,s+sizeof(uint16_t))));
		
		//Now, if this length is larger than the new one, then free the extra, if it is
		//	less than the new one then reallocate
		if (sizeold > size) {
			//The old size is larger than the new size, so we free the extra portion
			ndb_free(db,ndb_ploc(db,s+sizeof(uint16_t))+size+1,sizeold-size);
			//Now the allocated size is correct
		} else if (sizeold < size) {
			//The oldsize is less than the new size - so we free the entire old string,
			//	and create a new one
			ndb_free(db,ndb_ploc(db,s+sizeof(uint16_t)),sizeold+1);
			ndb_ploc(db,s+sizeof(uint16_t)) = ndb_alloc(db,size+1);
		}
	} else {
		//This is the first allocation, so we just allocate a new area
		ndb_ploc(db,s+sizeof(uint16_t)) = ndb_alloc(db,size+1); //We leave room for the \0
	}
	
	//All allocations are in order, we can now set the string
	strcpy(ndb_pstr(db,ndb_ploc(db,s+sizeof(uint16_t))),res);
	
	//Now s is at the location of the string allocation. We new hash the string ans set the hash
	ndb_pu16(db,s) = strhash(res);
}

//------------------------------------------------------------------------------------
//Extracting data from a table row


//Get the size in bytes of a table row
NDB_SIZE ndb_dtSize(nDB* db, NDB_LOC tbl) {
	uint8_t fnum;
	assert(db && tbl);
	
	fnum = ndb_otsNum(db,tbl);	//Gets the number of fields
	
	assert(fnum);	//Can't have 0 fields!
	
	//Return the size of all the fields put together
	return ndb_dtSizeTo(db,tbl,fnum);
	
}

//Creates and deletes table in data slot of node
void ndb_dataMk(nDB* db, NDB_LOC loc, uint8_t tbl) {
	NDB_LOC t = ndb_tableLoc(db,tbl);
	NDB_SIZE s;
	assert(db && loc && tbl && t && !ndb_odData(db,loc));
	
	//First, we set the table number at the location
	ndb_odTable(db,loc) = tbl;
	
	
	//Now, if the size of the table is less than or equal to NDB_LOC, we just put the
	//	data in the spot! If that is the case, we set the 0th boolean to 1
	s = ndb_dtSize(db,t);
	
	if (s) {	//If the size-get did not fail
		if (s <= sizeof(NDB_LOC)) {
			//The size is less, so we don't need to allocate, we just set the "fit" flag
			ndb_odBoolSet(db,loc,0,1);
		} else {
			//The size is larger, so we allocate!
			ndb_odData(db,loc) = ndb_alloc(db,s);
		}
	} else {
		ndb_log(NDB_LTBL,"data: Failed to create - invalid table\n");
	}
	
	//Aight, we made it, it is created
}

void ndb_dataDel(nDB* db, NDB_LOC loc, NDB_LOC tbl) {
	NDB_LOC t = ndb_tableLoc(db,tbl);
	NDB_SIZE s;
	assert(db && loc && tbl && t && !ndb_odData(db,loc));
	
	//First, we set the table number at the location
	ndb_odTable(db,loc) = tbl;
	
	
	//Now, if the size of the table is less than or equal to NDB_LOC, we just clean
	//	the data in the spot, if not, we do a free
	s = ndb_dtSize(db,t);
	
	if (s > sizeof(NDB_LOC)) {
		//Since the size is larger than the allocation, we free!
		ndb_free(db,ndb_odData(db,loc),s);
	
	} else if (!s) {
		//TODO: We can check allocation based on boolean
		ndb_log(NDB_LTBL,"data: Waring, failed to get tblsize, data might not be freed\n");
	}
	
	//Now we clear the table num, boolean and data
	ndb_odData(db,loc) = 0;
	ndb_odBoolSet(db,loc,0,0);
	ndb_odTable(db,loc) = 0;
}
//---------------------------------------------------------------------------------------
//Gets and sets variables in a table, given values
void ndb_dataSetU8(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field,uint8_t p) {
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return;
		case NDB_TU8:	//Since we are returning a number, just convert
			ndb_dsu8(db,loc,tbl,field,p);
			break;
		case NDB_TU16:
			ndb_dsu16(db,loc,tbl,field,p);
			break;
		case NDB_TU32:
			ndb_dsu32(db,loc,tbl,field,p);
			break;
		case NDB_TU64:
			ndb_dsu64(db,loc,tbl,field,p);
			break;
		case NDB_TSTR:
			//Converts number to string
			ndb_dsstr(db,loc,tbl,field,ndb_itoa(p));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
	}
}

void ndb_dataSetU16(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field,uint16_t p) {
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return;
		case NDB_TU8:	//Since we are returning a number, just convert
			ndb_dsu8(db,loc,tbl,field,(uint8_t)p);
			break;
		case NDB_TU16:
			ndb_dsu16(db,loc,tbl,field,p);
			break;
		case NDB_TU32:
			ndb_dsu32(db,loc,tbl,field,p);
			break;
		case NDB_TU64:
			ndb_dsu64(db,loc,tbl,field,p);
			break;
		case NDB_TSTR:
			//Converts number to string
			ndb_dsstr(db,loc,tbl,field,ndb_itoa(p));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
	}
}

void ndb_dataSetU32(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint32_t p) {
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return;
		case NDB_TU8:	//Since we are returning a number, just convert
			ndb_dsu8(db,loc,tbl,field,p);
			break;
		case NDB_TU16:
			ndb_dsu16(db,loc,tbl,field,p);
			break;
		case NDB_TU32:
			ndb_dsu32(db,loc,tbl,field,p);
			break;
		case NDB_TU64:
			ndb_dsu64(db,loc,tbl,field,p);
			break;
		case NDB_TSTR:
			//Converts number to string
			ndb_dsstr(db,loc,tbl,field,ndb_itoa(p));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
	}
}

void ndb_dataSetU64(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, uint64_t p) {
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return;
		case NDB_TU8:	//Since we are returning a number, just convert
			ndb_dsu8(db,loc,tbl,field,p);
			break;
		case NDB_TU16:
			ndb_dsu16(db,loc,tbl,field,p);
			break;
		case NDB_TU32:
			ndb_dsu32(db,loc,tbl,field,p);
			break;
		case NDB_TU64:
			ndb_dsu64(db,loc,tbl,field,p);
			break;
		case NDB_TSTR:
			//Converts number to string
			ndb_dsstr(db,loc,tbl,field,ndb_itoa(p));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
	}
}

void ndb_dataSetS(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field, char* p) {
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return;
		case NDB_TU8:	//Since we are returning a number, just convert
			ndb_dsu8(db,loc,tbl,field,atoi(p));
			break;
		case NDB_TU16:
			ndb_dsu16(db,loc,tbl,field,atoi(p));
			break;
		case NDB_TU32:
			ndb_dsu32(db,loc,tbl,field,atoi(p));
			break;
		case NDB_TU64:
			ndb_dsu64(db,loc,tbl,field,atoi(p));
			break;
		case NDB_TSTR:
			ndb_dsstr(db,loc,tbl,field,p);
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
	}
}
//--------------------------------------------------------------------------------------
//GETS
uint8_t ndb_dataGetU8(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	uint8_t res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return 0;
		case NDB_TU8:	//Since we are returning a number, just convert
			res = (uint8_t)ndb_dgu8(db,loc,tbl,field);
			break;
		case NDB_TU16:
			res = (uint8_t)ndb_dgu16(db,loc,tbl,field);
			break;
		case NDB_TU32:
			res = (uint8_t)ndb_dgu32(db,loc,tbl,field);
			break;
		case NDB_TU64:
			res = (uint8_t)ndb_dgu64(db,loc,tbl,field);
			break;
		case NDB_TSTR:
			//Try to convert the string to number
			//	If conversion fails, just returns 0
			res = (uint8_t)atoi(ndb_dgstr(db,loc,tbl,field));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return 0;
	}
	return res;
}

uint16_t ndb_dataGetU16(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	uint16_t res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return 0;
		case NDB_TU8:	//Since we are returning a number, just convert
			res = (uint16_t)ndb_dgu8(db,loc,tbl,field);
			break;
		case NDB_TU16:
			res = (uint16_t)ndb_dgu16(db,loc,tbl,field);
			break;
		case NDB_TU32:
			res = (uint16_t)ndb_dgu32(db,loc,tbl,field);
			break;
		case NDB_TU64:
			res = (uint16_t)ndb_dgu64(db,loc,tbl,field);
			break;
		case NDB_TSTR:
			//Try to convert the string to number
			//	If conversion fails, just returns 0
			res = (uint16_t)atoi(ndb_dgstr(db,loc,tbl,field));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return 0;
	}
	return res;
}

uint32_t ndb_dataGetU32(nDB* db, NDB_LOC loc,NDB_LOC tbl, uint8_t field) {
	uint32_t res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return 0;
		case NDB_TU8:	//Since we are returning a number, just convert
			res = (uint32_t)ndb_dgu8(db,loc,tbl,field);
			break;
		case NDB_TU16:
			res = (uint32_t)ndb_dgu16(db,loc,tbl,field);
			break;
		case NDB_TU32:
			res = (uint32_t)ndb_dgu32(db,loc,tbl,field);
			break;
		case NDB_TU64:
			res = (uint32_t)ndb_dgu64(db,loc,tbl,field);
			break;
		case NDB_TSTR:
			//Try to convert the string to number
			//	If conversion fails, just returns 0
			res = (uint32_t)atoi(ndb_dgstr(db,loc,tbl,field));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return 0;
	}
	return res;
}

uint64_t ndb_dataGetU64(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	uint64_t res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return 0;
		case NDB_TU8:	//Since we are returning a number, just convert
			res = (uint64_t)ndb_dgu8(db,loc,tbl,field);
			break;
		case NDB_TU16:
			res = (uint64_t)ndb_dgu16(db,loc,tbl,field);
			break;
		case NDB_TU32:
			res = (uint64_t)ndb_dgu32(db,loc,tbl,field);
			break;
		case NDB_TU64:
			res = (uint64_t)ndb_dgu64(db,loc,tbl,field);
			break;
		case NDB_TSTR:
			//Try to convert the string to number
			//	If conversion fails, just returns 0
			res = (uint64_t)atoi(ndb_dgstr(db,loc,tbl,field));
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return 0;
	}
	return res;
}

char* ndb_dataGetS(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	char* res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return NULL;
		case NDB_TU8:	//Convert the numbers to strings
			res = ndb_itoa(ndb_dgu8(db,loc,tbl,field));
			break;
		case NDB_TU16:
			res = ndb_itoa(ndb_dgu16(db,loc,tbl,field));
			break;
		case NDB_TU32:
			res = ndb_itoa(ndb_dgu32(db,loc,tbl,field));
			break;
		case NDB_TU64:
			res = ndb_itoa(ndb_dgu64(db,loc,tbl,field));
			break;
		case NDB_TSTR:
			res = ndb_dgstr(db,loc,tbl,field);
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return NULL;
	}
	return res;
}


uint32_t ndb_dataHash(nDB* db, NDB_LOC loc, NDB_LOC tbl, uint8_t field) {
	uint32_t res;
	uint8_t type = ndb_tfieldType(db,tbl,field);
	assert(db && loc && tbl);
	//Now, we create the conversion code
	switch (type) {
		case 0:
			ndb_log(NDB_LTBL,"DATA: Field type is NULL. Error.\n");
			ndb_errSet(db,15);
			return 0;
		case NDB_TU8:	//Since we are returning a number, just convert
			res = (uint32_t)ndb_dgu8(db,loc,tbl,field);
			break;
		case NDB_TU16:
			res = (uint32_t)ndb_dgu16(db,loc,tbl,field);
			break;
		case NDB_TU32:
			res = (uint32_t)ndb_dgu32(db,loc,tbl,field);
			break;
		case NDB_TU64:
			res = (uint32_t)ndb_dgu64(db,loc,tbl,field);
			break;
		case NDB_TSTR:
			//Get the string hash
			res = (uint32_t)ndb_dgshash(db,loc,tbl,field);
			break;
		default:
			ndb_log(NDB_LTBL,"DATA: Unrecognized field type. Error.\n");
			ndb_errSet(db,15);
			return 0;
	}
	return res;
}

//EOF

