/**************************************************************************************

                                     Root List

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "rootList.h"

#include "allocator.h"
#include "offsets.h"

//The indices used to make the rootList
#include "treeIndex.h"
#include "listIndex.h"

//-------------------------------------------------------------------------------------
//Internal Functions - not exposed 

NDB_LOC ndb_rlSwitchMk(nDB* db) {
	assert(db);
	
	ndb_log(NDB_LSTR,"RLIST: Allocating root page\n");
		
	return ndb_alloc(db,NDB_RLSIZE);
}

//-------------------------------------------------------------------------------------

//Add to the root index - l is the location, and the uint is the type of thing, node or link.
void ndb_rlAdd(nDB* db, NDB_LOC l, uint8_t nol) {
	NDB_LOC tblIndex;
	
	assert(db && l);
	assert(nol < NDB_RLSNUM);
	
	ndb_log(NDB_LSTR,"RLIST: Registering element of type %u\n",nol);
	
	if (!ndb_ohIndex(db)) {
		//The index does not exist yet
		ndb_ohIndex(db) = ndb_rlSwitchMk(db);
		if (!ndb_ohIndex(db)) {
			ndb_log(NDB_LSTR,"rlst: Allocation failed! Aborting.\n");
			return;
		}
	}
	//The rootList exists, now check the tabletree
	
	if (!ndb_orlGet(db,ndb_ohIndex(db),nol)) {
		//The table tree does not exist
		ndb_orlGet(db,ndb_ohIndex(db),nol) = ndb_tiMk(db,0,0);
		if (!ndb_orlGet(db,ndb_ohIndex(db),nol)) {
			ndb_log(NDB_LSTR,"rlst: tree creation failed! Aborting.\n");
			return;
		}
	}
	//The tableTree exists - now we check for the tableList
	tblIndex = ndb_orlGet(db,ndb_ohIndex(db),nol);
	
	
	if (!ndb_tiLoc(db,tblIndex,ndb_odTable(db,l))) {
		//The table's listIndex doesn't exist yet
		ndb_tiAdd(db,tblIndex,ndb_odTable(db,l),ndb_liMk(db,sizeof(NDB_LOC),0,0));
		if (!ndb_tiLoc(db,tblIndex,ndb_odTable(db,l))) {
			ndb_log(NDB_LSTR,"rlst: Failed to create listIndex for table\n");
			return;
		}
	}
	
	tblIndex = ndb_liAdd(db,ndb_tiLoc(db,tblIndex,ndb_odTable(db,l)));
	
	if (!tblIndex) {
		ndb_log(NDB_LSTR,"rlst: Failed to add to list\n");
		return;
	}
	
	//Write to the location
	ndb_ploc(db,tblIndex) = l;
}

//Get the list index for the given thing and table
NDB_LOC ndb_rlGetli(nDB* db, uint8_t nol, uint8_t tbl) {
	assert(db);
	assert(nol < NDB_RLSNUM);
	
	if (ndb_ohIndex(db)) {
		//An index exists
		if (ndb_orlGet(db,ndb_ohIndex(db),nol)) {
			//The tree exists
			return ndb_tiLoc(db,ndb_orlGet(db,ndb_ohIndex(db),nol),tbl);
		}
	}
	return 0;
}

//Iterates through the listIndex - it allows on-check deletion for faster
//	node table switching
NDB_LOC ndb_rlIterate(ndb_iter* i, uint8_t tbl) {
	NDB_LOC res;
	assert(i && i->db);
	do {
		res = ndb_liIterate(i);
		
		if (!res) return 0;	//The iteration is over
		if (ndb_ploc(i->db,res)!=0 && ndb_odTable(i->db,ndb_ploc(i->db,res))!=tbl) {
			//We have the wrong table in place - we wrase the reference, the user
			//	must have changed the table type
			ndb_ploc(i->db,res) = 0;
		}
	} while (!ndb_ploc(i->db,res));
	return ndb_ploc(i->db,res);
}

//EOF

