/**************************************************************************************

                                      HeapStruct
                               Methods to access Data

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "heapstruct.h"

#include "ndb_pg.h"	//Includes the page modification routines
#include <string.h>

//-------------------------------------------------------------------------------------
//Heap-specific functions

NDB_PGNUM heap_addPage_(nDB* db) {
	NDB_PGNUM p;
	ndb_page* pg;
	assert(db);
	if ((pg = ndb_getPage(db,ndb_newPage(db)))) {	//Get a new page, if the page exists
		ndb_putlP(pg,NDB_LHEAP);	//Put the label
		//Now the page is ready for action!
		
		assert(pg->page);	//I don't even know how this could not be the case
		p = pg->page;
		
		ndb_freePage(pg);
		return p;	//It does not include the offset
	}
	return 0;	//A fail
}
#define heap_addPage heap_addPage_
//-------------------------------------------------------------------------------------
//HEAPSTRUCT

NDB_FLOC ndb_mkHeap_(nDB* db) {
	assert(db->pagesize > 17);	//That is the minimum page size heap accepts
	assert(db && !db->heap);	//Make sure that the values are defined
	db->heap = heap_addPage(db);	//Not much to do, huh.
	return db->heap;
}

/*
	Right now, the allocator creates an entire new page for each heap allocation. This needs
	to be modified for small payloads to write to the end of the "heapstruct", but for some reason
	I really don't feel like doing that now.
*/
NDB_FLOC ndb_heapAlloc_(nDB* db, uint32_t size) {
	ndb_page* pg;
	//uint16_t loc;
	uint32_t sizeleft;
	assert(db && db->heap && size);	//The values have to be defined
	
	//Okay, so if the size to allocate is larger than 1/2 of a page, then we just allocate one for
	//	it, since with larger data it is easier to manage based on pages - and it is probably text
	//	so that simplifies reallocation, since a whole page can be much more easily resized if it has
	//	its own "thread" of memory - the pages allow the allocator to playaround with pages.
	//if (size > db->pagesize/2) {
		//Here, we create a whole new sequence of pages for a large item. Since the amount needed is 
		//	indeterminate, we cake the allocation a linked-list-style structure
		NDB_PGNUM prev = 0;	//The previous page link
		
		sizeleft = size;	//The amount of page room still necessary for object allocation
		
		//Create allocation space from the back, so move back page by page until we have enough room
		while (sizeleft > db->pagesize - 2*sizeof(uint32_t)-sizeof(NDB_PGNUM)-1) {
			//We need another page
			NDB_PGNUM n = heap_addPage(db);
			
			if (n && (pg = ndb_gcPage(db,n,NDB_LHEAP))) {
				ndb_pgDone(pg);			//gcPage sets a read lock on the page
				ndb_pgWriteLock(pg);	//Write lock the page
				
				ndb_pgwpg(pg,db->pagesize-NDB_PGNSIZE,prev);	//Write the link to the next page
				ndb_pgWriteUnlock(pg);
				
				ndb_freePage(pg);	//The page is not necessary anymore
				
				prev = n;	//Iterate to the next page link
				
				//We make sure that we don't accidentally asplode the uint
				if (db->pagesize - sizeof(NDB_PGNUM)-1 > sizeleft) sizeleft = 0;
				else sizeleft -= db->pagesize - sizeof(NDB_PGNUM)-1;
				
			} else return 0;	//Failed to allocate somewhere
		}
		
		//Now all the extension pages were allocated. This means that there is a allocation surplus of:
		size += db->pagesize - 2*sizeof(uint32_t)-sizeof(NDB_PGNUM)-1 - sizeleft;
		
		//We create the last page: the first one
		if ((pg = ndb_gcPage(db,heap_addPage(db),NDB_LHEAP))) {
			ndb_pgDone(pg);			//gcPage sets a read lock on the page
			ndb_pgWriteLock(pg);	//Write lock the page
			
			ndb_pgwpg(pg,db->pagesize-NDB_PGNSIZE,prev);	//Write the link to the next page
			
			prev = pg->page;	//Writes the location of this page to prev for later use
			
			//Now we diverge from the previous: we write the allocation header
			ndb_pgwu32(pg,1,size);				//Size is the allocation amt
			ndb_pgwu32(pg,1+sizeof(uint32_t),0);	//None of it is used ATM
			
			ndb_pgWriteUnlock(pg);
			
			ndb_freePage(pg);	//The page is not necessary anymore
			
		} else return 0;	//Failed to allocate somewhere
		
		return prev*db->pagesize+1;	//Returns the page location of the allocated unit
		
	//} else {
		//The size is small, so we don't allocate dedicated pages for the object
		//This time, we allocate at the end of the "heap struct" so that whole pages are not wasted
		//	for small pieces of data
		
		//If we are to create a new page: if the size is larger than room on the current page
		//TODO: This
	//}
	
	//It returns no matter what
	//return 0;	//It faled, error was set in subroutine	
}

char ndb_heapCopy_(nDB* db, NDB_FLOC to, NDB_FLOC from, uint32_t size, uint32_t offset) {
	return 0;
}

NDB_FLOC ndb_heapWrite_(nDB* db, NDB_FLOC loc,void* object,uint32_t size,uint32_t offset) {
	//Write values TO the heap
	ndb_page* pg = ndb_gcPage(db,ndb_locPage(db,loc),NDB_LHEAP);
	if (pg) {
		uint32_t s = ndb_pgru32(pg,ndb_locLoc(db,loc));	//Obtain the allocation size
		
		//Now we check if the already allocated size is large enough to hold the new data
		if (s < size + offset) {
			if (s >= db->pagesize - ndb_locLoc(db,loc)-2*sizeof(uint32_t)-sizeof(NDB_PGNSIZE) &&
				(s-(db->pagesize-ndb_locLoc(db,loc)-2*sizeof(uint32_t)-sizeof(NDB_PGNSIZE)))%(db->pagesize-1-sizeof(NDB_PGNSIZE))) {
				//It can link its own page!
				//We traverse the link structure until we get to the desired spot
				
			} else {
				//It does not have its own pagestuff, so we need to reallocate
				NDB_FLOC loc2;
			
				//The size is not large enough, so reallocate
				ndb_pgDone(pg);
				ndb_freePage(pg);
			
				//We now have a new location
				loc2 = ndb_heapAlloc(db,size+offset+NDB_HEAP_RESIZE_OFFSET);	//Allocate new memory
				ndb_heapCopy(db,loc2,loc,s,0);	//Copy the memory to the new location
			
				//Set the new memory location, and the size of the total
				s = size+offset+NDB_HEAP_RESIZE_OFFSET;
				loc = loc2;
			}
		}
	}
	return 0;
}

uint8_t ndb_heapRead_(nDB* db,NDB_FLOC loc,void* object, uint32_t size, uint32_t offset) {
	//Read values on the heap
	return 0;
}

char ndb_heapCmp(nDB* db, NDB_FLOC heaploc, uint32_t offset,void* object, uint32_t size) {
	//Compare two binary values at a location
	return 0;
}

uint8_t ndb_heapStrcmp(nDB* db, NDB_FLOC hloc, char* string) {
	//Compare strings in the heap
	return ndb_heapCmp(db,hloc,0,string,strlen(string));
}
	
uint32_t ndb_heapSize(nDB* db, NDB_FLOC loc) {
	//Get the page associated with the location, and read the size off of the header
	//	of the location
	ndb_page* pg = ndb_gcPage(db,ndb_locPage(db,loc),NDB_LHEAP);
	if (pg) {
		//The page is now open for reading
		uint32_t s = ndb_pgru32(pg,ndb_locLoc(db,loc)+sizeof(uint32_t));	//Size is after the allocation
		ndb_pgDone(pg);
		ndb_freePage(pg);
		return s;
	}
	return 0;
}

//EOF

