/**************************************************************************************

                                    Error Handling
                                     Definitions

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_ERR
#define NDB_ERR 1

#include "ndb_def.h"

/*
This does all error handling for the database. It obtains error codes from within the
constituents of nDB and contains their translation into strings. By calling it, the user
can find out both what error number happened and its translation into English. Furthermore,
*/
uint8_t ndb_peekerr(nDB* db);			//Gets the error number. Returns 0 if no error detected.
uint8_t ndb_geterr(nDB* db);			//Same as above, but resets the error to 0
char* ndb_geterrStr(nDB* db);			//Returns the string associated with the current error
char* ndb_geterrString(uint8_t e);		//Given an error number, returns associated string
void ndb_errinit(nDB* db);				//Initializes error handling
void ndb_errclose(nDB* db);				//Shuts down error handling

//The user should call ndb_seterr()!!
void ndb_seterr_(nDB* db, uint8_t e);	//Sets the error to the given number
#define ndb_seterr ndb_seterr_

//-------------------------------------------------------------------------------------
//Error Definitions
//	These are the actual error numbers

#define NDBERR_CORRUPTED	1
#define NDBERR_NULL			2
#define NDBERR_MEMORY		3
#define NDBERR_UNKNOWN		4
#define NDBERR_CACHE		5

//-------------------------------------------------------------------------------------
//VERBOSE ERROR DISPLAY
//	For logging, it displays error in the log file

#if defined(NDB_EVER) && NDB_HASLOG

#undef ndb_seterr
#define ndb_seterr(a,b) ndbver_seterr(a,b,__FILE__,__LINE__)

void ndbver_seterr(nDB* db, uint8_t e,char* loc,int line);

#endif

#endif
