/**************************************************************************************

                                      HeapStruct
                               Methods to access Data

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "heapstruct.h"

#include "ndb_pg.h"	//Includes page access functions
#include "error.h"
#include <string.h>

//-------------------------------------------------------------------------------------
//Heap-specific functions

NDB_PGNUM heap_addPage_(nDB* db) {
	NDB_PGNUM p;
	ndb_page* pg;
	assert(db);
	if ((pg = ndb_getPage(db,ndb_newPage(db)))) {	//Get a new page, if the page exists
		ndb_putlP(pg,NDB_LHEAP);	//Put the label
		//Now the page is ready for action!
		
		assert(pg->page);	//I don't even know how this could not be the case
		p = pg->page;
		
		ndb_freePage(pg);
		return p;	//It does not include the offset
	}
	return 0;	//A fail
}
#define heap_addPage heap_addPage_

//Gets the page and associated header
ndb_page* heap_gpg(nDB* db, NDB_FLOC *f, NDB_HSIZE* totsize, NDB_HSIZE* usedsize) {
	ndb_page* pg;
	assert(db && f && totsize && usedsize);
	
	pg = ndb_gcPage(db,ndb_locPage(db,*f),NDB_LHEAP);
	if (pg) {
		*totsize	= ndb_pgrhs(pg,ndb_locLoc(db,*f));
		*usedsize	= ndb_pgrhs(pg,ndb_locLoc(db,*f)+sizeof(NDB_HSIZE));
	}
	return pg;
}

//Finds the offset page location
ndb_page* heap_findOffset(nDB* db, NDB_FLOC *f, ndb_page* p, NDB_HSIZE offset,NDB_HSIZE totsize, NDB_HSIZE usedsize) {
	assert(db && f && p);
	assert(offset < totsize);	//We need to be sure that we are not supposed to go beyond the heapstruct's bounds
		
	while (1) {	//I spit in the face of infinite loops
		//If it fits on the current page, go to the advised location. If not, obtain the next page and repeat
		if (db->pagesize - ndb_locLoc(db,*f)-NDB_PGNSIZE >=offset || db->pagesize -ndb_locLoc(db,*f)>= totsize) {
			*f += offset;	//Set the location on the page that the offset is achieved
			return pg;		//Returns the page that the offset is on
		} else {
			//We lower the size by the current page
			totsize	-= db->pagesize - ndb_locLoc(db,*f) - NDB_PGNSIZE;	//The amount of room on the page
			offset	-= db->pagesize - ndb_locLoc(db,*f) - NDB_PGNSIZE;	//The offset left to move by
			
			//Get the next page link - and convert it straight to location form for ease of use
			*f = ndb_pgrpg(pg,db->pagesize-NDB_PGNSIZE)*db->pagesize+1;
			
			ndb_pgDone(pg);
			ndb_freePage(pg);
			
			assert(*f > 1);	//The 0th page should never come up in this situation
			
			//Get the associated page
			if (!(pg = ndb_gcPage(db,ndb_locLoc(db,*f),NDB_LHEAP))) break;
			//	I remember that breaks were bad programming practice. So I included one just to piss
			//	some CS teachers off.
		}
	}
	return NULL;
}
/*
//Returns the size left - it is dangerous to use without knowing exactly how it is going to affect 
//	the code! It does stuff in an unconventional way, sizeleft is to be modified ONLY by the getNext function,
//	with the starting number being total size - offset+loc-1
ndb_page* heap_getNext(nDB* db, ndb_page* p, NDB_FLOC* f,NDB_HSIZE* sizeleft) {
	assert(db && p && f && sizeleft);
	
	//If there is still data left
	if (sizeleft) {
		//First, set the amount to decrease by this page
		if (db->pagesize - ndb_locLoc(db,*f)-NDB_PGNSIZE > *sizeleft) *sizeleft =0;
		else sizeleft = db->pagesize - ndb_locLoc(db,*f)-NDB_PGNSIZE;
		
		//Get the next page link - and convert it straight to location form for ease of use
		*f = ndb_pgrpg(pg,db->pagesize-NDB_PGNSIZE)*db->pagesize+1;
		
		//Free the page
		ndb_pgDone(pg);
		ndb_freePage(pg);
		
		//Return the page gotten
		return ndb_gcPage(db,ndb_locLoc(db,*f),NDB_LHEAP);
	}
	
	return NULL;
}*/

//-------------------------------------------------------------------------------------

NDB_PGNUM ndb_mkHeap(nDB* db) {
	assert(db->pagesize > 1+2*sizeof(NDB_HSIZE)+NDB_PGNSIZE);	//That is the minimum page size heap accepts
	assert(db && !db->heap);	//Make sure that the values are defined
	db->heap = heap_addPage(db);	//Not much to do, huh.
	return db->heap;
}

//TODO: the allocation code is so long and confusing - cleaning it up is definitely a priority
NDB_FLOC ndb_heapAlloc_(nDB* db, NDB_HSIZE size) {
	ndb_page* pg;
	NDB_HSIZE sizeleft = size;
	assert(db && db->heap && size);	//These values all have to be defined!
	
	//If the allocation size is larger than NDB_HASIZE of a page, it gets its own page, if not,
	//	it is appended to the end of the current allocation unit.
	
	if (size > NDB_HASIZE*db->pagesize) {
		//The size is large - so we allocate new pages
		
		//Create the page number holders
		NDB_PGNUM cur;
		NDB_PGNUM prev = 0;	//Link to the previous page
		
		//Creates the allocation space from the back - to simplify page linking.
		//	While the size left is larger than the amount the first page can hold,
		//	keep creating new pages.
		
		//Addon: The last "link" in the chain will not be used, and thus can be used
		//	as storage space. We assume that the space can immediately be used.
		assert(sizeleft >= sizeof(NDB_PGNUM));	//We assume that the size allocated is
			//at least as large as a link - otherwise the current algorithm fails
			//TODO: Fix this bug
		
		sizeleft -= sizeof(NDB_PGNUM);
		
		while (sizeleft > (db->pagesize-2*sizeof(NDB_HSIZE)-sizeof(NDB_PGNUM)-1)) {
			cur = heap_addPage(db);
			
			if (cur && (pg = ndb_gcPage(db,cur,NDB_LHEAP))) {
				ndb_pgDone(pg);		//We don't need to read from the page - we write to it!
				ndb_pgWriteLock(pg);//We now set the write lock
				
				ndb_pgwpg(pg,db->pagesize-NDB_PGNSIZE,prev);
				
				ndb_pgWriteUnlock(pg);
				ndb_freePage(pg);
				
				prev = cur;	//The current page is now the 'previous' one.
				
				//Now we subtract the size necessary from the page. In order not to
				//	reverse-overflow the uint, we make sure that it is not negative, since
				//	the first page can use less space than normal pages, by 2*sizeof(NDB_HSIZE)
				if (sizeleft > (db->pagesize - sizeof(NDB_PGNUM)-1))
					sizeleft = sizeleft - (db->pagesize - sizeof(NDB_PGNUM)-1);
				else
					sizeleft = 0;	//It would overflow, so just set it to 0
				
				
			} else return 0;	//If the page does not exist or there was an error,
				//	we should return an error rather than trying to continue
		}
		
		//The extra pages were created. Now we make a new page for the header.
		
		//The only problem remaining is that the size of the allocation might now be
		//	larger than requested - we change the size to accomodate this
		//add the extra space to the size, so:
		//add to size: page room available on new page - page room originally allocated
		size += (db->pagesize - 2*sizeof(NDB_HSIZE)-sizeof(NDB_PGNUM)-1) - sizeleft;
		
		//And now we create the last page in the same way as in the loop
		cur = heap_addPage(db);
			
		if (cur && (pg = ndb_gcPage(db,cur,NDB_LHEAP))) {
			ndb_pgDone(pg);		//We don't need to read from the page - we write to it!
			ndb_pgWriteLock(pg);//We now set the write lock
			
			ndb_pgwpg(pg,db->pagesize-NDB_PGNSIZE,prev);
			
			//Write the header alongside the nextPage link, if such exists
			ndb_pgwhs(pg,1,size);
			ndb_pgwhs(pg,1+sizeof(NDB_HSIZE),0);
			
			ndb_pgWriteUnlock(pg);
			ndb_freePage(pg);
			
			prev = cur;	//The current page is now the 'previous' one.
			
			//Now we subtract the size necessary from the page. In order not to
			//	reverse-overflow the uint, we make sure that it is not negative, since
			//	the first page can use less space than normal pages, by 2*sizeof(NDB_HSIZE)
			if (sizeleft > (db->pagesize - sizeof(NDB_PGNUM)-1))
				sizeleft = sizeleft - (db->pagesize - sizeof(NDB_PGNUM)-1);
			else
				sizeleft = 0;	//It would overflow, so just set it to 0
			
			
		} else return 0;	//If the page does not exist or there was an error,
			//	we should return an error rather than trying to continue
		
		return cur*db->pagesize+1;	//Return the link to the newly allocated space
	} else {
		//The size is small, it is added to the end of the current allocation for heap
		NDB_FLOC hl = db->heap;
		
		//First, assert that there is enough room on the current page
		//	Previous allocations should have included code that 
		//	should make sure that this is always the case
		assert(db->pagesize-ndb_locLoc(db,db->heap)>=2*sizeof(NDB_HSIZE)+NDB_PGNSIZE);
		
		//Opens the current location page
		if ((pg = ndb_gcPage(db,ndb_locPage(db,db->heap),NDB_LHEAP))) {
			ndb_pgDone(pg);			//We need not read the page
			ndb_pgWriteLock(pg);	//We will only be writing
			
			//If the current allocation will make it necessary to create a new page, make it here
			if (size > (db->pagesize - ndb_locLoc(db,db->heap))) {
				//We allocate the new page and create the link
				NDB_PGNUM prev = heap_addPage(db);
				
				if (!prev) {ndb_pgWriteUnlock(pg);ndb_freePage(pg);return 0;}
				
				//Switch the heap to a new location
				db->heap = db->pagesize*prev+db->pagesize+1-(size-(db->pagesize-ndb_locLoc(db,db->heap)));
				
				ndb_pgwpg(pg,db->pagesize-sizeof(NDB_PGNSIZE),prev);
				
				
				//TODO: There is a possible bug here with small page sizes and large NDB_HASIZE
				//	where even after allocating new page there is not enough room left in the new page
				//	for the next allocation. This assertion is to notify if such an error occurs
				assert(size-(db->pagesize-ndb_locLoc(db,db->heap))<db->pagesize-(2*sizeof(NDB_HSIZE)+sizeof(NDB_PGNSIZE)+1));
				
			} else {
				if (db->pagesize-ndb_locLoc(db,db->heap)-size<2*sizeof(NDB_HSIZE)+sizeof(NDB_PGNUM)) {
					//The room left after the allocation is not large enough for future allocations:
					//	change the current allocation size to fill up the current location
					size += db->pagesize-ndb_locLoc(db,db->heap)-size;
				}
				//Switch the heap pointer to its new location
				db->heap = db->heap + size;
			} 
			
			ndb_pgwhs(pg,ndb_locLoc(db,db->heap),size);					//Writes heap allocation size
			ndb_pgwhs(pg,ndb_locLoc(db,db->heap)+sizeof(NDB_HSIZE),0);	//The currently allocated amt
			
			ndb_pgWriteUnlock(pg);
			ndb_freePage(pg);
			
			return hl;
		}
	}
	
	return 0;
}
NDB_FLOC ndb_heapWrite_(nDB* db, NDB_FLOC loc,void* object,NDB_HSIZE size,NDB_HSIZE offset)	{
	NDB_HSIZE tot,used;
	ndb_page* pg = heap_gpg(db,&loc,&tot,&used);
	
	if (pg) {
		if (offset + size < tot) {
			
		}
	}
	return 0;
}
uint8_t ndb_heapRead_(nDB* db,NDB_FLOC loc,void* object,NDB_HSIZE size,NDB_HSIZE offset) {
	NDB_HSIZE tot,used;
	ndb_page* pg = heap_gpg(db,&loc,&tot,&used);
	
	if (pg && offset+size < used) {
		//If everything is in order
		
	}
	return 0;
}
char ndb_heapCopy_(nDB* db, NDB_FLOC to,NDB_HSIZE offsetTo,NDB_FLOC from,NDB_HSIZE offsetFrom,NDB_HSIZE size) {
	return 0;
}
char ndb_heapCmp(nDB* db, NDB_FLOC heaploc, NDB_HSIZE offset,void* object, NDB_HSIZE size) {
	return 0;
}
uint8_t ndb_heapStrcmp(nDB* db, NDB_FLOC hloc, char* string) {
	return ndb_heapCmp(db,hloc,0,string,strlen(string));
}

uint32_t ndb_heapSize(nDB* db, NDB_FLOC loc) {
	//Get the page associated with the location, and read the size off of the header
	//	of the location
	ndb_page* pg = ndb_gcPage(db,ndb_locPage(db,loc),NDB_LHEAP);
	if (pg) {
		//The page is now open for reading
		NDB_HSIZE s = ndb_pgrhs(pg,ndb_locLoc(db,loc)+sizeof(NDB_HSIZE));	//Size is after the allocation
		ndb_pgDone(pg);
		ndb_freePage(pg);
		return s;
	}
	return 0;
}

//EOF

