/**************************************************************************************

                                  NDB Logging File

**************************************************************************************/
//Tab Width: 4 Character:\t

#define NDB_MVER_H		//We don't want the header to screw up memory logging
#include "ndb_log.h"
#include "ndb_def.h"	//Want to find out if log is active

#include <stdarg.h>

#if NDB_HASLOG

//By default, the file is stdout - -1 is automatically rerouted to it.
FILE* logfile = (FILE*)(1);
//By default it prints all outputs 
uint8_t loglevel = 0;

FILE* ndb_getLogFile() {
	if (logfile ==(FILE*)1) logfile=stdout;
	return logfile;
}
void ndb_setLogFile(FILE* f) {
	logfile = f;
}

uint8_t ndb_getLogLevel() {
	return loglevel;
}
void ndb_setLogLevel(uint8_t level) {
	loglevel = level;
}

void ndb_log(uint8_t level,char* str,...) {
	if (logfile ==(FILE*)1) logfile=stdout;
	
	if (level >= loglevel && logfile) {
		va_list a_list;
		va_start(a_list,str);
		vfprintf(logfile,str,a_list);
		va_end(a_list);
		#ifdef NDBLOG_FLUSH
		fflush(logfile);
		#endif
	}
}

#endif

#if defined(NDB_MVER) && NDB_HASLOG

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>

void* m_malloc(size_t s,char* loc,int line) {
	void* ptr = malloc(s);
	ndb_log(NDB_MVER,"\n(MALLOC Ptr:%-16lu| Size:%-5zu | %s:%d )",(uint64_t)ptr,s,loc,line);
	return ptr;
}

void m_free(void* ptr,char* loc,int line) {
	ndb_log(NDB_MVER,"\n(FREE   Ptr:%-16lu| %s:%d )",(uint64_t)ptr,loc,line);
	free(ptr);
}

void m_memset(void* ptr, int n, size_t len, char* loc,int line) {
	ndb_log(NDB_MVER,"\n(MEMSET Ptr:%-16lu| Size:%-5zu | %s:%d )",(uint64_t)ptr,len,loc,line);
	memset(ptr,n,len);
}
void* m_realloc(void* pointer,size_t s, char* loc,int line) {
	void* ptr = realloc(pointer,s);
	ndb_log(NDB_MVER,"\n(RALLOC Ptr:%lu -> %lu | NewSize:%-5zu | %s:%d )",(uint64_t)pointer,(uint64_t)ptr,s,loc,line);
	return ptr;
}
void* m_calloc(size_t s, int num, char* loc,int line) {
	void* ptr = calloc(s,num);
	ndb_log(NDB_MVER,"\n(CALLOC Ptr:%-16lu| Size:%-5zu | %s:%d )",(uint64_t)ptr,s,loc,line);
	return ptr;
}

#endif
//EOF


