/**************************************************************************************

                         Allocation for Database Structures
                                    Definitions

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_ALLOC
#	define NDB_ALLOC

#include "ndb_def.h"	//Includes the necessary defintions

NDB_LOC ndb_alloc_(nDB* db, NDB_SIZE size);
	//Allocates the given amount from the file, returning a file offset for the allocated
	//	section.
void ndb_free_(nDB* db, NDB_LOC loc, NDB_SIZE size);
	//Frees the given size from the location - the allocator does not remember how much it
	//	allocated, so this must be passed. This allows for lowering the allocation size easily.

NDB_LOC ndb_ralloc_(nDB* db, NDB_LOC old, NDB_SIZE oldsize, NDB_SIZE newsize);
	//This is not a powerful reallocator - nDB doesn't really use it for database structures, it is
	//	used for blob data - it literally allocates a new region and copies the old data to it.

//These allow quicker access to the locations
//	These are macros, not real functions, so they allow usage from right and left,
//	which can be a bit confusing at first
#define	ndb_ptr(db,loc) ((void*)((db)->map+(loc)))
#define	ndb_ploc(db,loc) (*(NDB_LOC*)((db)->map+(loc)))
#define	ndb_psize(db,loc) (*(NDB_SIZE*)((db)->map+(loc)))
#define	ndb_pu8(db,loc) (*(uint8_t*)((db)->map+(loc)))
#define	ndb_pu16(db,loc) (*(uint16_t*)((db)->map+(loc)))
#define	ndb_pu32(db,loc) (*(uint32_t*)((db)->map+(loc)))
#define	ndb_pu64(db,loc) (*(uint64_t*)((db)->map+(loc)))
#define	ndb_p32(db,loc) (*(int32_t*)((db)->map+(loc)))

#if NDB_LOG

#define ndb_alloc(a,b) ndb_alloc__(a,b,NDB_LD)
#define ndb_free(a,b,c) ndb_free__(a,b,c,NDB_LD)
#define ndb_ralloc(a,b,c,d) ndb_ralloc__(a,b,c,d,NDB_LD)

NDB_LOC ndb_alloc__(nDB* db, NDB_SIZE size,NDB_LF);
void ndb_free__(nDB* db, NDB_LOC loc, NDB_SIZE size,NDB_LF);
NDB_LOC ndb_ralloc__(nDB* db, NDB_LOC old, NDB_SIZE oldsize, NDB_SIZE newsize,NDB_LF);

#else

#define ndb_alloc ndb_alloc_
#define ndb_free ndb_free_
#define ndb_ralloc ndb_ralloc_

#endif

#endif
//EOF

