/**************************************************************************************

                             Error Handling & Logging
                                 Implementation

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "log_err.h"	//Include self

#include <errno.h>		//Allows to obtain error information
#include <string.h>		//For strerror()

//-------------------------------------------------------------------------------------
//Logging:

#if NDB_LOG

#include <stdarg.h>	//Allows for a multiargument function

//The log has 2 global variables: The file to write to, and the level at which to
//	start accepting signals
FILE* logfile = (FILE*)1;	//The log file is stdout by default
uint8_t loglevel = 0;		//By default it prints all outputs

FILE* ndb_logGetFile() {
	if (logfile==(FILE*)1) logfile = stdout;
	return logfile;
}

void ndb_logSetFile(FILE* f) {
	logfile = f;
}

uint8_t ndb_logGetLevel() {
	return loglevel;
}

void ndb_logSetLevel_(uint8_t level,NDB_LF) {
	loglevel = level;
	ndb_ilog(NDB_LLOG,"%8.8s:%-4d| LOGLEVEL %u\n",f,l,level);
}

void ndb_ilog(uint8_t level,char* str, ...) {
	if (logfile ==(FILE*)1) logfile=stdout;
	
	if (level >= loglevel && logfile) {
		va_list a_list;
		va_start(a_list,str);
		vfprintf(logfile,str,a_list);
		va_end(a_list);
		#ifdef NDB_LOGFLUSH
		fflush(logfile);
		#endif
	}
}

void ndb_log_(NDB_LF,uint8_t level, char* str, ...) {
	if (logfile ==(FILE*)1) logfile=stdout;		//This is a copy of the ilog code with one printf added
	
	if (level >= loglevel && logfile) {
		va_list a_list;
		fprintf(logfile,"%8.8s:%-4d| ",f,l);	//The file and line
		va_start(a_list,str);
		vfprintf(logfile,str,a_list);
		va_end(a_list);
		fprintf(logfile,"\n");
		#ifdef NDB_LOGFLUSH
		fflush(logfile);
		#endif
	}
}

#endif

//-------------------------------------------------------------------------------------
//Error Management:

//Error string definitions
#define ESLEN 3
char* errstring[ESLEN] = {
	"Hakuna Matata",		//0: No errors
	"Not Enough Memory",	//1: Memory
	"Failed to open file"	//2: FileError
};

void ndb_errSetdb_(nDB* db, uint8_t e) {
	assert(db && e < ESLEN);
	
	db->err = errstring[e];
		//Set the error to the database error string
}

void ndb_errSet_(nDB* db) {
	assert(db);
	
	//We got some type of error, we set it to the error string
	db->err = strerror(errno);
}

void ndb_errSetcustom_(nDB* db, char* err) {
	assert(db);
	
	db->err = err;	//Set the error string
}
char* ndb_errPeek(nDB* db) {
	assert(db);
	return db->err;
}

char* ndb_errGet(nDB* db) {
	char* tmp;
	assert(db);
	tmp = ndb_errPeek(db);
	db->err=0;
	return tmp;
}

uint8_t ndb_errGetdb(nDB* db) {
	uint8_t i = ESLEN-1;
	for (;i>0;i--)
		if (errstring[i]==db->err) break;
	return i;
}

#if NDB_LOG

void ndb_errSetdb__(nDB* db, uint8_t e,NDB_LF) {
	ndb_errSetdb_(db,e);
	ndb_ilog(NDB_LERR,"%8.8s:%-4d| NDBERR: %-25s\n",f,l,ndb_errPeek(db));
}
void ndb_errSet__(nDB* db,NDB_LF) {
	ndb_errSet_(db);
	ndb_ilog(NDB_LERR,"%8.8s:%-4d| ERR: %-29s\n",f,l,ndb_errPeek(db));
}
void ndb_errSetcustom__(nDB* db,char* err,NDB_LF) {
	ndb_errSetcustom_(db,err);
	ndb_ilog(NDB_LERR,"%8.8s:%-4d| CERR: %-29s\n",f,l,ndb_errPeek(db));
}
#endif

//EOF

