/**************************************************************************************

                                  nDB Misc Functions

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "misc.h"
#include "allocator.h"	//For pointer manipulation simplifiers

//------------------------------------------------------------------------------------
//Setting Booleans

uint8_t ndb_miscGetbool(nDB* db, NDB_LOC loc, uint8_t num) {
	assert(db && loc && num < 8);	//The boolean must be within the specified range
	return (ndb_pu8(db,loc) & (1<<num));
}
void ndb_miscSetbool_(nDB* db, NDB_LOC loc, uint8_t num, uint8_t b) {
	assert(db && loc && num < 8);	//The boolean must be within the specified range
	if (b) ndb_pu8(db,loc)= (ndb_pu8(db,loc) | (1<<num));
	else ndb_pu8(db,loc) = (ndb_pu8(db,loc) & ~(1<<num));

}
void ndb_miscFlipbool_(nDB* db, NDB_LOC loc, uint8_t num) {
	assert(db && loc && num < 8);	//The boolean must be within the specified range
	ndb_pu8(db,loc) = (ndb_pu8(db,loc) ^ (1<<num));
}

//------------------------------------------------------------------------------------
//Millisecond counter

//WARNING: SYSTEM-DEPENDENT!
#ifdef __linux__

#include <sys/time.h>

uint64_t ndb_millis() {
	//The only problem is that it is not guaranteed to have 1 ms accuracy. The approximation
	//	will have to do, since it should be close enough for most database applications
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return (uint64_t)tv.tv_sec * 1000 + tv.tv_usec / 1000;
}
#endif

#ifdef _WIN32

#include <windows.h>

//his is actually one of the cool things about the windows compiler
#pragma comment(lib,"Winmm")

//#error Millis not implemented yet on Windows!
uint64_t ndb_millis() {
	ndb_log(NDB_LMSG,"Warning: ndb_millis returns time since system start on Windows, rather than epoch!");
	return timeGetTime();
}
#endif

#if NDB_LOG

void ndb_miscSetbool__(nDB* db, NDB_LOC loc, uint8_t num, uint8_t b,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SETBOOL %u at %lu to %i\n",f,l,num,loc,(b?1:0));
	ndb_miscSetbool_(db,loc,num,b);
}
void ndb_miscFlipbool__(nDB* db, NDB_LOC loc, uint8_t num,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| FLPBOOL %u at %lu\n",f,l,num,loc);
	ndb_miscFlipbool_(db,loc,num);
}

#endif

//EOF

