/**************************************************************************************

                                  nDB Definitions

**************************************************************************************/
//Tab Width: 4 Character:\t
/*
These are compile-time parameters, along with descriptions of what they do
Comment them out if you do not wish them to be activated
*/

//The amount of allocation-units to resize each time
#define NDB_RESIZE 5

//The Log is a debugging tool - it prints the actions of the database to a file
//	of your choice.
#define NDB_LOG 1

//Whether or not to flush the log after writing each line - only used if log is enabled
#define NDB_LOGFLUSH

//The logLevels of different parts of the database
//	These cannot be commented out, they can only be changed
#define NDB_LERR 200		//Errors
#define NDB_LMSG 199		//Messages

#define NDB_LLOG 1			//Log settings
#define NDB_LMEM 1			//Memory actions
#define NDB_LHDR 2			//Header write
#define NDB_LALC 3			//Allocator level
#define NDB_LSTR 5			//The database structure level
#define NDB_LOBJ 6			//The database object level
#define NDB_LQUE 7			//The query level
#define NDB_LBAS 10			//Base level

#define NDB_LTODO 0			//logging of todos

//END OF COMPILE-TIME PARAMETERS
//-------------------------------------------------------------------------------------

#ifndef NDB_DEF
#	define NDB_DEF

#define NDB_VER 1	//The version of nDB

//-------------------------------------------------------------------------------------
//Database Includes

#include <stdlib.h>	//All of nDB uses the C standard library
#include <stdint.h>	//Fixed integer sizes for platform independence
#include <assert.h>	//The code makes extensive use of assertions for debugging

#ifdef _WIN32
#include <Windows.h>
#endif

//-------------------------------------------------------------------------------------
//Definitions

#define NDB_LOC uint64_t	//A location pointer
#define NDB_SIZE uint32_t	//A size pointer
#define NDB_LINK uint8_t	//A link size

//The Database Object - contains all of the database's information
typedef struct {
#ifdef _WIN32
	HANDLE file;			//Windows uses a different object type for files
	HANDLE fmap;			//Aaand windows also uses some extra stuff for file access.
#else
	int file;				//The file descriptor
#endif
	uint8_t readonly :1;	//Whether or not the file is readonly
	
	char* map;				//The file's memory map, it is the way the file
							//	is accessed and modified
	uint64_t size;			//The size of the memory map
	
	//File and map manipulation functions
	char (*resize)(void*,NDB_LOC);	//This function resizes to the given size
	void (*close)(void*);				//This function closes the file and map
		//Since there can be arbitrary methods of mapping, like mmap and even just a 
		//	malloc for in-memory, the database has to form a sort of 'virtualization'
	
	//Error Handling
	char* err;
		//There is no such thing as error number - the error string pointers are what
		//	errors are defined as in nDB
	
	//Note that any database-wide information is included in the database itself
} nDB;

//-------------------------------------------------------------------------------------
//nDB's iterator

//The iter struct is used for iterating data - a pointer to an iter struct is passed
typedef struct {
	nDB* db;		//The database object
	uint64_t data;	//Data that the iterator holds
	NDB_LOC loc;		//A page link
} ndb_iter;

//-------------------------------------------------------------------------------------
//Logging & Error

#ifndef NDB_LOG
#	define NDB_LOG 0
#endif
	//If Log was not defined yet, it is clearly not desired, so we set it to 0

#include "log_err.h"
	//The logging and error handling functions are present for the whole database

#endif
//EOF

