/**************************************************************************************

                              nDB Edge Sequence Struct

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "seqpg.h"
#include "edge.h"
#include "allocator.h"	//For pointer and allocator

//------------------------------------------------------------------------------------
//Sequence page manipulation functions

NDB_LOC ndb_seqpgMk_(nDB* db,NDB_LOC node) {
	NDB_LOC ptr;
	assert(db);	//The database must exist 
	ptr = ndb_alloc(db, NDB_SEQSIZE);
	if (ptr) {
		ndb_seqpgSetnode(db,ptr,node);
	}
	return ptr;
}

NDB_LOC ndb_seqpgGetprev(nDB* db, NDB_LOC s) {
	assert(db && s);
	return ndb_ploc(db,s+sizeof(NDB_LOC));
}
NDB_LOC ndb_seqpgGetnext(nDB* db, NDB_LOC s) {
	assert(db && s);
	return ndb_ploc(db,s+2*sizeof(NDB_LOC));
}
NDB_LOC ndb_seqpgGetnode(nDB* db, NDB_LOC s) {
	assert(db && s);
	return ndb_ploc(db,s);
}
uint64_t ndb_seqpgGetnum(nDB* db, NDB_LOC s) {
	assert(db && s);
	return ndb_pu64(db,s+3*sizeof(NDB_LOC));
}

void ndb_seqpgSetprev_(nDB* db, NDB_LOC s, NDB_LOC e) {
	assert(db && s);
	ndb_ploc(db,s+sizeof(NDB_LOC)) = e;
}
void ndb_seqpgSetnext_(nDB* db, NDB_LOC s, NDB_LOC e) {
	assert(db && s);
	ndb_ploc(db,s+2*sizeof(NDB_LOC)) = e;
}
void ndb_seqpgSetnode_(nDB* db, NDB_LOC s, NDB_LOC e) {
	assert(db && s);
	ndb_ploc(db,s) = e;
}
void ndb_seqpgSetnum_(nDB* db, NDB_LOC s, uint64_t e) {
	assert(db && s);
	ndb_ploc(db,s+3*sizeof(NDB_LOC)) = e;
}

NDB_LOC ndb_seqpgGetedge(nDB* db, NDB_LOC s, uint8_t num) {
	assert(db && s && num < NDB_SEQEDGES);
	return (s+3*sizeof(NDB_LOC)+sizeof(uint64_t)+num*NDB_EDGESIZE);
}

#if NDB_LOG

NDB_LOC ndb_seqpgMk__(nDB* db,NDB_LOC node,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SEQPG: Create page\n",f,l);
	return ndb_seqpgMk_(db,node);
}
void ndb_seqpgSetprev__(nDB* db, NDB_LOC s, NDB_LOC e,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SEQPG: Set pg %lu: 'prev'<-%lu \n",f,l,s,e);
	return ndb_seqpgSetprev_(db,s,e);
}
void ndb_seqpgSetnext__(nDB* db, NDB_LOC s, NDB_LOC e,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SEQPG: Set pg %lu: 'next'<-%lu \n",f,l,s,e);
	return ndb_seqpgSetnext_(db,s,e);
}
void ndb_seqpgSetnode__(nDB* db, NDB_LOC s, NDB_LOC e,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SEQPG: Set pg %lu: 'node'<-%lu \n",f,l,s,e);
	return ndb_seqpgSetnode_(db,s,e);
}
void ndb_seqpgSetnum__(nDB* db, NDB_LOC s, uint64_t e,NDB_LF) {
	ndb_ilog(NDB_LSTR,"%8.8s:%-4d| SEQPG: Set pg %lu: 'num'<-%lu \n",f,l,s,e);
	return ndb_seqpgSetnum_(db,s,e);
}

#endif

//EOF

