/**************************************************************************************

                                    nDB Edge Sequence

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "sequence.h"

#include "seqpg.h"	//Includes the necessary structures
#include "node.h"
#include "edge.h"

//------------------------------------------------------------------------------------

uint64_t ndb_edgegetNum(nDB* db, NDB_LOC e) {
	uint8_t num;
	assert(db && e);
	num = ndb_edgeGetsnum(db,e);	//We have the edge's number in sequence page
	
	//Now, we need the location of the sequence page 
	e = e - num*NDB_EDGESIZE - NDB_SEQHEADSIZE;
	
	return num + ndb_seqpgGetnum(db,e);
}

//------------------------------------------------------------------------------------

void ndb_seqMk(nDB* db, NDB_LOC n) {
	assert(db && n);
	ndb_nodesetSeq(db,n,ndb_seqpgMk(db,n));
}

NDB_LOC ndb_seqGet(nDB* db, NDB_LOC n, uint64_t num) {
	ndb_iter i;
	assert(db && n);
	ndb_seqIterinit(db,n,&i);	//Initialize the iterator
	
	return ndb_seqIterget(&i,num);
}
NDB_LOC ndb_seqAdd(nDB* db,NDB_LOC n,NDB_LOC from,NDB_LOC to) {
	ndb_iter i;
	assert(db && n);
	ndb_seqIterinit(db,n,&i);
	return ndb_seqIteradd(&i,from,to);
}

void ndb_seqIterinit(nDB* db, NDB_LOC n,ndb_iter* i) {
	assert(db && n && i);
	
	//Sets the necessary parts
	i->db = db;
	i->data = 0;
	i->loc = ndb_nodegetSeq(db,n);

}


NDB_LOC ndb_seqIterget(ndb_iter* i, uint64_t num) {
	assert(i && i->db);
	
	//Make sure that there is at least one edge in the whole thing
	if (ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc)) == 0) return 0;
	
	//While the number we are supposed to get is ahead of us
	while (i->data/NDB_SEQEDGES < num/NDB_SEQEDGES && ndb_seqpgGetnext(i->db,i->loc)) {
		//The number we are at is still not on the same page as the one we want, and 
		//	the sequence has another page, so we move to that page
		i->loc = ndb_seqpgGetnext(i->db,i->loc);	//Move the iterator location
		i->data = ndb_seqpgGetnum(i->db,i->loc);	//Change the number we are at
	}
	
	//We either hit the last page, are pages ahead of the desired number, or we are on
	//	the correct page. Now we move back if we need to do so
	if (i->data/NDB_SEQEDGES > num/NDB_SEQEDGES) {
		//We need to move back - there are two choices now, we can either start over from
		//the beginning, or iterate back - we now choose which way is faster
		if (i->data/NDB_SEQEDGES - num/NDB_SEQEDGES > num/NDB_SEQEDGES) {
			//It is simpler to start over - so we do
			i->data = 0;	//Reset data and the sequence page
			i->loc = ndb_nodegetSeq(i->db,ndb_seqpgGetnode(i->db,i->loc));
			
			return ndb_seqIterget(i,num);	//call from the top
		}
		
		//So it is faster to move backwards, so let's do it!
		while (i->data/NDB_SEQEDGES > num/NDB_SEQEDGES && ndb_seqpgGetprev(i->db,i->loc)) {
			//We are still ahead of the desired page
			i->loc = ndb_seqpgGetprev(i->db,i->loc);
			i->data = ndb_seqpgGetnum(i->db,i->loc);
		}
	}
	
	//At this point we are either at the right page, or it failed somewhere along the way
	
	//We check if it failed to get the page
	if (i->data/NDB_SEQEDGES != num/NDB_SEQEDGES) return 0;
	
	
	//We are at the right page! Now check if the desired edge exists
	//	We do this by making sure that the "from" link exists
	if (!ndb_edgeGetfrom(i->db,ndb_seqpgGetedge(i->db,i->loc,num%NDB_SEQEDGES))) return 0;
	
	//So the desired edge exists! We move the iterator into position
	i->data = num;
	
	return ndb_seqpgGetedge(i->db,i->loc,num%NDB_SEQEDGES);
}

NDB_LOC ndb_seqIteradd_(ndb_iter* i,NDB_LOC from, NDB_LOC to) {
	assert(i && i->db);	//This needs to exist!
	
	//Okay - so there are two options that we check for - first, 
	//	is that there is NOTHING in the sequence, and second is that
	//	there is something in the sequence:
	if (ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc)) == 0) {
		//If the sequence is empty, we make the first edge in sequence
		
		//Get the first edge
		NDB_LOC e = ndb_seqpgGetedge(i->db,i->loc,0);
		
		//Set the sequence number
		ndb_nodesetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc),1);
		
		//Create the edge, with the number 0
		ndb_edgeMk(i->db,e,0,from,to);
		
		//Return the edge location
		return e;
		
	} else {	//Else: There are already edges in the sequence, so we can search them
		NDB_LOC npg;	//A variable used as placeholder
		
		//We now make sure that we are at the last link, and if not, make the iterator 
		//	go to the last link
		if (i->data+1 != ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc))) {
			//We are not at the last node! Go to it!
			//	The -1 is there because sequence numbers are offset by one!
			ndb_seqIterget(i,ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc))-1);
		}
		
		//Now we have to be at the last link, since that is guaranteed by the previous
		//	statement - we now check the veracity of this claim
		assert(i->data+1 == ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc)));
		
		//Now, we iterate the amount of sequence pages, and move the iterator by one,
		//	while checking if we need to add a page to the sequence
		if ((++i->data) % NDB_SEQEDGES == 0) {
			//We need to add a page!
			npg = ndb_seqpgMk(i->db,ndb_seqpgGetnode(i->db,i->loc));
			assert(npg);	//make sure this exists
			
			ndb_seqpgSetnext(i->db,i->loc,npg);	//Sets the next page of the series
			
			ndb_seqpgSetprev(i->db,npg,i->loc);	//Sets the next page's data
			ndb_seqpgSetnum(i->db,npg,i->data);
			
			i->loc = npg;	//Set the iterator's location to the new page
		}
		
		//Set the number in sequence
		ndb_nodesetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc),i->data+1);
		
		//Get the new edge
		npg = ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES);
		
		//Set the basics of the edge
		ndb_edgeMk(i->db,npg,i->data%NDB_SEQEDGES,from,to);
		
		//Return the pointer
		return npg;
	}
}

//Iterates through the sequence one by one
NDB_LOC ndb_seqIterate(ndb_iter* i) {
	assert(i && i->db);
	
	//Make sure that there is at least one edge in the whole thing
	if (ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc)) == 0) return 0;
	
	//If we can't move forward any more
	if (i->data+1 == ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc))) return 0;
	
	//We now check if the link is on the next page
	if ((++i->data)%NDB_SEQEDGES == 0) {
		i->loc = ndb_seqpgGetnext(i->db,i->loc);
		assert(i->loc);	//The page HAS to exist
	}
	
	//The iterator is set correctly now - and we know that an edge HAS to exist!
	assert(ndb_edgeGetfrom(i->db,ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES))) ;
	
	//Return that edge
	return ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES);
}

NDB_LOC ndb_seqIterateback(ndb_iter* i) {
	assert(i && i->db);
	
	//Make sure that there is at least one edge in the whole thing
	if (ndb_nodegetSeqnum(i->db,ndb_seqpgGetnode(i->db,i->loc)) == 0) return 0;
	
	//If we can't move back any more
	if (i->data == 0) return 0;
	
	//We now check if the link is on the previous page
	if ((i->data--)%NDB_SEQEDGES == 0) {
		i->loc = ndb_seqpgGetprev(i->db,i->loc);
		assert(i->loc);	//If it doesn't exist, there is a bug in the database!
	}
	
	//The iterator is set correctly now - and we know that an edge HAS to exist!
	assert(ndb_edgeGetfrom(i->db,ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES))) ;
	
	//Return that edge
	return ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES);
}

NDB_LOC ndb_seqItergetcur(ndb_iter* i) {
	assert(i && i->db);
	return ndb_seqpgGetedge(i->db,i->loc,i->data%NDB_SEQEDGES);
}

#if NDB_LOG

NDB_LOC ndb_seqIteradd__(ndb_iter* i, NDB_LOC from,NDB_LOC to,NDB_LF) {
	ndb_ilog(NDB_LOBJ,"%8.8s:%-4d| SEQ: Add edge to sequence\n",f,l);
	return ndb_seqIteradd_(i,from,to);
}

#endif

//EOF

