/**************************************************************************************

                                   nDB Basic Tests

**************************************************************************************/
//Tab Width: 4 Character:\t
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#if defined(__GNUC__) && defined(linux)
#include <mcheck.h>
#endif

//Global, did everything go correctly
char haderror = 0;
char flogg = 0;
char color = 0;


#include "nDB/offsets.h"

//-------------------------------------------------------------------
//Test of Core
#include "nDB/core.h"
char* core() {
	nDB a;
	nDB* ndb;
	nDB* db;

	//First we create 4 files - one with a failed id, one with a large version
	//	and one that is good - and we make sure that checkFile works correctly
	FILE* f = fopen("nDB.test.1.db","w");
	fwrite("nqB",4,1,f);
	putc(1,f);
	fclose(f);
	f = fopen("nDB.test.2.db","w");
	fwrite("nDB",4,1,f);
	putc(255,f);
	fclose(f);
	f = fopen("nDB.test.3.db","w");
	fwrite("nDB",4,1,f);
	putc(1,f);
	putc(42,f);
	fclose(f);
	f = fopen("nDB.test.4.db","w");
	fwrite("nDB",4,1,f);
	putc(1,f);
	putc(1,f);
	putc(342,f);
	fclose(f);
	f = fopen("nDB.test.5.db","w");
	fwrite("nDB",4,1,f);
	putc(1,f);
	putc(1,f);
	putc(ndb_getComp(),f);
	fclose(f);
	
	//Check the files
	if (!ndb_checkFile("nDB.test.1.db")) return "1: Checking database validity failed";
	if (!ndb_checkFile("nDB.test.2.db")) return "2: Checking database validity failed";
	if (!ndb_checkFile("nDB.test.3.db")) return "3: Checking database validity failed";
	if (!ndb_checkFile("nDB.test.4.db")) return "4: Checking database validity failed";
	if (ndb_checkFile("nDB.test.5.db")) return "5: Checking database validity failed";
	
	//Clear out the files
	remove("nDB.test.1.db");
	remove("nDB.test.2.db");
	remove("nDB.test.3.db");
	remove("nDB.test.4.db");
	remove("nDB.test.5.db");
	
	//So checkfile works
	
	//Now we check makeHeader
	a.map = malloc(50);
	a.size=50;
	
	ndb_hwrite(&a);	//Writes the header
	
	if (strcmp(a.map,"nDB")) return "6: Header Creation Failed";
	if (a.map[4]!=NDB_VER) return "7: Header version mismatch";
	if (a.map[5]!=NDB_VER) return "8: Header version mismatch";
	
	//Now we set each thing individually, checking to make sure they don't f us up
	if (ndb_ohIndex(&a)) return "9: Database variables not null";
	if (ndb_ohTable(&a)) return "10: Database variables not null";
	if (ndb_ohTrash(&a)) return "11: Database variables not null";
	
	//The end variable is already set
	
	//Sets the values
	ndb_ohIndex(&a) = 32434235;
	ndb_ohTable(&a) = 32434234;
	ndb_ohTrash(&a) = 32434233;
	ndb_ohEnd(&a) = 32434232;
	
	//Checks to make sure all is well
	if (32434235 != ndb_ohIndex(&a)) return "12: Database variables not set";
	if (32434234 != ndb_ohTable(&a)) return "13: Database variables not set";
	if (32434233 != ndb_ohTrash(&a)) return "14: Database variables not set";
	if (32434232 != ndb_ohEnd(&a)) return "15: Database variables not set";
	
	free(a.map);
	
	//Create memory database
	db = ndb_open(NULL);
	
	if (!db) return "16: Failed to create database";
	
	ndb_ohTrash(db) = 32343;
	
	ndb = ndb_copy(db,"testdb.db");
	
	if (!ndb) return "17: Failed to copy database";
	
	if (ndb_ohTrash(ndb) != 32343) return "18: Copying database failed";
	
	ndb_close(db);
	ndb_close(ndb);
	
	if (ndb_open_v2(&a,"testdb.db",0)) return "19: Failed to repoen database";
	
	if (ndb_ohTrash(&a) != 32343) return "20: Getting data from database failed";
	
	ndb_close_v2(&a);
	remove("testdb.db");
	
	return NULL;
}

//-------------------------------------------------------------------
//Test of Mapper
#include "nDB/mapper.h"

char* mapper() {
	//We create a database
	nDB a;
	void *trollface;
	a.map = NULL;	//NULL the associated parts
	a.file= 0;
	a.size= 100;
	
	//First try a memory database
	ndb_mapMem(&a);
	if (!a.map || a.size<100) return "1: Failed to create a map of the right size";
	if (*((uint64_t*)(a.map+90))) return "2: Map was initially not empty!";
	*((uint64_t*)(a.map+90)) = 865744534;
	if (*((uint64_t*)(a.map+90)) != 865744534) return "3: Map writing failed!";
	
	trollface = malloc(3);	//Allocate memory right after it (hopefully) so that it is forced to copy
	
	ndb_mapMemResize(&a,150);
	
	free(trollface);
	
	if (!a.map || a.size<150) return "4: Resizing map failed";
	if (*((uint64_t*)(a.map+90)) != 865744534) return "5: Data was not copied to new map";
	if (*((uint64_t*)(a.map+120))!=0) return "6:New map locations are non-NULL";
	
	ndb_mapMemClose(&a);
	
	//Now we map a REAL file
	a.size = 50;
	a.readonly = 0;
	if (ndb_mapFile(&a,"test.db")) return "7: File mapping failed";
	if (!a.map || a.size <50) return "8: Mapping does not set values";
	
	if (*((uint64_t*)(a.map+30))) return "9: Map was initially not empty!";
	*((uint64_t*)(a.map+30)) = 865744534;
	if (*((uint64_t*)(a.map+30)) != 865744534) return "10: Map writing failed!";
	if (ndb_mapFileResize(&a,15023)) return "11: Map resize failed";
	if (!a.map || a.size<5023) return "12: Resizing map failed at values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "13: Data was not copied to new map";
	if (*((uint64_t*)(a.map+14020))!=0) return "14:New map locations are non-NULL";
	*((uint64_t*)(a.map+14900)) = 865744034;
	ndb_mapFileClose(&a);
	
	if (ndb_mapFile(&a,"test.db")) return "15: File mapping failed";
	if (!a.map || a.size <15023) return "16: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "17: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "18: Map read failed!";
	ndb_mapFileClose(&a);
	
	
	//Now that RW was tested, we can try out readonly access
	a.readonly = 1;
	if (ndb_mapFile(&a,"test.db")) return "19: File mapping failed";
	if (!a.map || a.size <15023) return "20: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "21: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "22: Map read failed!";
#ifndef _WIN32	//On windows, you can't do writing to readonly mappings, so this fails
	*((uint64_t*)(a.map+150)) = 4354;
	if (*((uint64_t*)(a.map+150)) != 4354) return "23: Map read failed!";
	/*Currently, mapFileResize can't resize read-only files, so this part is t3h fale
	if (ndb_mapFileResize(&a,50023)) return "24: Map resize failed";
	*((uint64_t*)(a.map+49000)) = 4355;
	if (*((uint64_t*)(a.map+150)) != 4354) return "25: Map read failed!";
	if (*((uint64_t*)(a.map+49000)) != 4355) return "26: Map read failed!";
	*/
	ndb_mapFileClose(&a);
	
	//Now make sure that it was NOT written to file!
	a.readonly =1;
	if (ndb_mapFile(&a,"test.db")) return "27: File mapping failed";
	if (!a.map || a.size <15023) return "28: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "29: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "30: Map read failed!";
	if (*((uint64_t*)(a.map+150)) != 0) return "31: Map read failed!";
	if (a.size >= 40000) return "32: Map file sizing incorrect!";
#endif
	ndb_mapFileClose(&a);
	
	remove("test.db");
	return 0;
}

//-------------------------------------------------------------------
//Test of Error handler
#include "nDB/error.h"

char* error() {
	nDB a;
	a.err = 0;
	
	if (ndb_errPeek(&a) != 0) return "1: err was initially non-NULL";
	ndb_errSet(&a,1);
	if (ndb_errPeek(&a) != 1) return "2: err Set failed to accomplish";
	if (ndb_errGet(&a) != 1) return "3: err Set failed to accomplish";
	if (ndb_errGet(&a) != 0) return "4: err reset failed to accomplish";
	return NULL;
}

//-------------------------------------------------------------------
//Allocator

#include "nDB/allocator.h"
char* allocator() {
	nDB* db = ndb_open("testbase.db");
	char* str = "Hello World!";
	NDB_LOC ptr = ndb_alloc(db,strlen(str)+1);
	
	strcpy(db->map+ptr,str);
	
	ndb_ohTable(db) = ptr;
	ptr = ndb_alloc(db,sizeof(uint64_t));
	*((uint64_t*)(db->map+ptr)) = 5463464523;
	ndb_ohIndex(db) = ptr;
	ptr = ndb_alloc(db,34564);
	
	if (ndb_ohIndex(db) + sizeof(uint64_t) != ptr) return "1: Allocator not giving correct answer";
	
	*(db->map+ptr+34000)=54;
	
	if (ndb_ohEnd(db) < 34564) return "2: Sizing failed";
	ndb_close(db);
	db = ndb_open("testbase.db");
	
	if (!(ptr = ndb_ohTable(db))) return "3: Memory set incorrectly";
	if (strcmp(db->map+ptr,"Hello World!")) return "4: Memory incorrectly set";
	
	if (!(ptr = ndb_ohIndex(db))) return "5: Memory set incorrectly";
	if (*((uint64_t*)(db->map+ptr)) != 5463464523) return "6: Memory failed to keep";
	
	ptr += sizeof(uint64_t);
	
	if (*(db->map+ptr+34000)!=54) return "7: Memory get failed";
	
	ndb_close(db);
	remove("testbase.db");
	return NULL;
}

//-------------------------------------------------------------------
//Test of offsets
#include "nDB/offsets.h"

char* offsets(nDB* db) {
	NDB_LOC l = ndb_alloc(db,1);	//Allocates a byte
	
	//Check the booleans
	if (ndb_boolGet(db,l,0)) return "1: Booleans not false";
	if (ndb_boolGet(db,l,1)) return "2: Booleans not false";
	if (ndb_boolGet(db,l,2)) return "3: Booleans not false";
	if (ndb_boolGet(db,l,3)) return "4: Booleans not false";
	if (ndb_boolGet(db,l,4)) return "5: Booleans not false";
	if (ndb_boolGet(db,l,5)) return "6: Booleans not false";
	if (ndb_boolGet(db,l,6)) return "7: Booleans not false";
	if (ndb_boolGet(db,l,7)) return "8: Booleans not false";
	
	ndb_boolSet(db,l,0,1);
	ndb_boolSet(db,l,5,0);
	ndb_boolSet(db,l,6,1);
	
	if (!ndb_boolGet(db,l,0)) return "9: Booleans incorrectly set";
	if (ndb_boolGet(db,l,5)) return "10: Booleans incorrectly set";
	if (!ndb_boolGet(db,l,6)) return "11: Booleans incorrectly set";
	if (ndb_boolGet(db,l,1)) return "12: Booleans not false";
	if (ndb_boolGet(db,l,2)) return "13: Booleans not false";
	if (ndb_boolGet(db,l,3)) return "14: Booleans not false";
	if (ndb_boolGet(db,l,4)) return "15: Booleans not false";
	if (ndb_boolGet(db,l,7)) return "16: Booleans not false";
	
	ndb_boolFlip(db,l,7);
	ndb_boolFlip(db,l,0);
	
	if (ndb_boolGet(db,l,0)) return "17: Booleans incorrectly set";
	if (ndb_boolGet(db,l,5)) return "18: Booleans incorrectly set";
	if (!ndb_boolGet(db,l,6)) return "19: Booleans incorrectly set";
	if (ndb_boolGet(db,l,1)) return "20: Booleans not false";
	if (ndb_boolGet(db,l,2)) return "21: Booleans not false";
	if (ndb_boolGet(db,l,3)) return "22: Booleans not false";
	if (ndb_boolGet(db,l,4)) return "23: Booleans not false";
	if (!ndb_boolGet(db,l,7)) return "24: Booleans incorrectly set";
	
	return NULL;
}

//-------------------------------------------------------------------
//Test of DataTables
#include "nDB/table.h"

char* tables(nDB* db) {
	uint8_t tbl = ndb_tableMk(db,5,"loller");
	NDB_LOC l = ndb_tableLoc(db,tbl);
	uint8_t num;
	if (tbl != 1) return "1: Failed to create table";
	if (ndb_tableFind(db,"loller")!=1) return "2: Failed to find table";
	if (!l) return "3: Failed to get table location";
	ndb_tfieldSet(db,l,0,1,"field1");
	ndb_tfieldSet(db,l,1,1,"field2-extralength");
	ndb_tfieldSet(db,l,2,2,"field3");
	ndb_tfieldSet(db,l,3,45,"field4");
	ndb_tfieldSet(db,l,4,4,"field5");
	num = ndb_tfieldFind(db,l,"field3");
	if (num!=2) return "3: Failed to find field";
	if (strcmp(ndb_tfieldName(db,l,num),"field3")) return "4: Failed to get field name";
	num = ndb_tfieldFind(db,l,"field2-extralength");
	if (num!=1) return "5: Failed to find field";
	if (strcmp(ndb_tfieldName(db,l,num),"field2-extralength")) return "6: Failed to get field name";
	num = ndb_tfieldFind(db,l,"field5");
	if (num!=4) return "7: Failed to find field";
	if (ndb_tfieldType(db,l,3)!=45) return "8: Failed to get field num";
	return NULL;
}

//-------------------------------------------------------------------
//Test of Data
#include "nDB/data.h"

char* data(nDB* db) {
	uint8_t tbl = ndb_tableMk(db,3,"ohai");
	NDB_LOC l = ndb_tableLoc(db,tbl);
	NDB_LOC l1;
	NDB_LOC l2;
	if (!tbl) return "1: Failed to create table";
	if (!l) return "2: Failed to get table location";
	ndb_tfieldSet(db,l,0,NDB_TU64,"int64");
	ndb_tfieldSet(db,l,1,NDB_TU16,"int16");
	ndb_tfieldSet(db,l,2,NDB_TSTR,"string");
	
	//Table is created and set!
	//Now we create locations, or table rows
	l1 = ndb_alloc(db,NDB_DATASIZE);
	l2 = ndb_alloc(db,NDB_DATASIZE);

	//Now we set the locations with our given data
	ndb_dataMk(db,l1,tbl);
	ndb_dataMk(db,l2,tbl);

	//We set the two locations, now we just set the data
	ndb_dataSetS(db,l1,l,2,"OHAI WORLD!");
	ndb_dataSetS(db,l2,l,2,"Shiiiiii!");

	ndb_dataSetU16(db,l1,l,1,4353);
	ndb_dataSetU16(db,l2,l,1,2343);

	ndb_dataSetU64(db,l1,l,0,43537965);
	ndb_dataSetU64(db,l2,l,0,23434353);

	//The data of the tables is set
	if (strcmp(ndb_dataGetS(db,l1,l,2),"OHAI WORLD!")) return "3: Failed to get String";
	if (strcmp(ndb_dataGetS(db,l2,l,2),"Shiiiiii!")) return "4: Failed to get String";

	if (ndb_dataGetU16(db,l1,l,1)!=4353) return "5: Failed to get int";
	if (ndb_dataGetU16(db,l2,l,1)!=2343) return "6: Failed to get int";

	if (ndb_dataGetU64(db,l1,l,0)!=43537965) return "7: Failed to get longint";
	if (ndb_dataGetU64(db,l2,l,0)!=23434353) return "8: Failed to get longint";
	
	//Okay, so the direct-getting of data succeeded. Now we test automatic data conversion
	if (strcmp(ndb_dataGetS(db,l2,l,0),"23434353")) return "9: Failed int->string conversion";
	if (ndb_dataGetU64(db,l1,l,1)!=4353) return "10: Failed int->longint conversion";
	
	//Now check set-conversions
	ndb_dataSetU64(db,l1,l,2,23434353);	//This is a string field
	
	//It should have saved as a string
	if (strcmp(ndb_dataGetS(db,l1,l,2),"23434353")) return "11: Failed to do SET string conversion";
	if (ndb_dataGetU64(db,l1,l,2)!=23434353) return "12: Failed string->longint conversion";
	return NULL;
}

//-------------------------------------------------------------------
//List Index
#include "nDB/listIndex.h"

char* lindex(nDB* db) {
	NDB_LOC e0,e1,e2,e3,e4,e5,e6,e7,e8,e9;
	ndb_iter i;
	NDB_LOC n = ndb_liMk(db,sizeof(NDB_LOC),0,0);
	
	e0 = ndb_liAdd(db,n);
	e1 = ndb_liAdd(db,n);
	e2 = ndb_liAdd(db,n);
	e3 = ndb_liAdd(db,n);
	e4 = ndb_liAdd(db,n);
	e5 = ndb_liAdd(db,n);
	e6 = ndb_liAdd(db,n);
	e7 = ndb_liAdd(db,n);
	e8 = ndb_liAdd(db,n);
	e9 = ndb_liAdd(db,n);
	
	if (!e0 || !e1 || !e2 || !e3 || !e4 || !e5 || !e6 || !e7 || !e8 || !e9)
		return "1: Adding to index failed";
	
	if (e0+sizeof(NDB_LOC) != e1) return "1.5: Incorrect Sizing!";
	
	if (ndb_liGetnum(db,n) != 10) return "2: Incorrect numbering";
	
	if (e3 != ndb_liGet(db,n,3)) return "3: Failed at getting";
	if (e0 != ndb_liGet(db,n,0)) return "4: Failed at getting";
	if (e9 != ndb_liGet(db,n,9)) return "5: Failed at getting";
	if (0 != ndb_liGet(db,n,10)) return "6: Failed at getting";
	
	//Now we try the iterator
	ndb_liIterinit(db,n,&i);
	
	if (ndb_liIterate(&i) !=e0) return "7: Failed at iterating";
	if (ndb_liIterate(&i) != e1) return "8: Failed at iterating";
	if (ndb_liIterate(&i) != e2) return "9: Failed at iterating";
	if (ndb_liIterate(&i) != e3) return "10: Failed at iterating";
	if (ndb_liIterate(&i) != e4) return "11: Failed at iterating";
	if (ndb_liIterate(&i) != e5) return "12: Failed at iterating";
	if (ndb_liIterate(&i) != e6) return "13: Failed at iterating";
	if (ndb_liIterate(&i) != e7) return "14: Failed at iterating";
	if (ndb_liIterate(&i) != e8) return "15: Failed at iterating";
	if (ndb_liIterate(&i) != e9) return "16: Failed at iterating";
	if (ndb_liIterate(&i) != 0) return "17: Failed at iterating";
	if (ndb_liIteratecur(&i) != e9) return "18: Failed at iterating";
	if (ndb_liIterateback(&i) != e9) return "18.5: Failed at iterating";
	if (ndb_liIterateback(&i) != e8) return "19: Failed at iterating";
	if (ndb_liIterateback(&i) != e7) return "20: Failed at iterating";
	if (ndb_liIterateback(&i) != e6) return "21: Failed at iterating";
	if (ndb_liIterateback(&i) != e5) return "22: Failed at iterating";
	if (ndb_liIterateback(&i) != e4) return "23: Failed at iterating";
	if (ndb_liIterateback(&i) != e3) return "24: Failed at iterating";
	if (ndb_liIterateback(&i) != e2) return "25: Failed at iterating";
	if (ndb_liIterateback(&i) != e1) return "26: Failed at iterating";
	if (ndb_liIterateback(&i) != e0) return "27: Failed at iterating";
	if (ndb_liIterateback(&i) != 0) return "28: Failed at iterating";
	
	
	if (e9 != ndb_liIterget(&i,9)) return "29: Failed at getting";
	
	if (e2 != ndb_liIterget(&i,2)) return "30: Failed at getting";
	
	if (e9 != ndb_liIterget(&i,9)) return "31: Failed at getting";
	
	if (e8 != ndb_liIterget(&i,8)) return "32: Failed at getting";
	if (e7 != ndb_liIterateback(&i)) return "33: Failed at iterating after get";
	return NULL;
}

//-------------------------------------------------------------------
//Tree Index
#include "nDB/treeIndex.h"

char* tindex(nDB* db) {
	
	NDB_LOC n = ndb_tiMk(db,0,0);
	
	ndb_tiAdd(db,n,4,434533);
	ndb_tiAdd(db,n,8,4342333);
	ndb_tiAdd(db,n,32,2234533);
	
	if (ndb_tiLoc(db,n,3)) return "1: Empty spot non-0";
	if (ndb_tiLoc(db,n,4)!=434533) return "2: Location set incorrectly";
	if (ndb_tiLoc(db,n,8)!=4342333) return "3: Location set incorrectly";
	if (ndb_tiLoc(db,n,32)!=2234533) return "4: Location set incorrectly";
	return NULL;
}

//-------------------------------------------------------------------
//Root List
#include "nDB/rootList.h"

char* rlist(nDB* db) {
	uint8_t tbl = ndb_tableMk(db,3,"rlist");
	NDB_LOC l = ndb_tableLoc(db,tbl);
	NDB_LOC l1;
	NDB_LOC l2;
	NDB_LOC tmp;
	ndb_iter i;

	if (!tbl) return "1: Failed to create table";
	if (!l) return "2: Failed to get table location";
	ndb_tfieldSet(db,l,0,NDB_TU64,"int64 field");
	ndb_tfieldSet(db,l,1,NDB_TU16,"int16 field");
	ndb_tfieldSet(db,l,2,NDB_TSTR,"string field");
	
	//We have created a table
	//Table is created and set!
	//Now we create locations, or table rows
	l1 = ndb_alloc(db,NDB_DATASIZE);
	l2 = ndb_alloc(db,NDB_DATASIZE);
	
	//We add the locations to the index
	ndb_rlAdd(db,l1,0);
	ndb_rlAdd(db,l2,0);
	
	if (ndb_rlGetli(db,1,2)) return "3: Getting has phony data";
	if (ndb_rlGetli(db,0,2)) return "4: Getting has phony data";
	tmp = ndb_rlGetli(db,0,0);
	if (!tmp) return "5: Getting has no data";
	
	ndb_liIterinit(db,tmp,&i);
	
	if (ndb_rlIterate(&i,0) != l1) return "6: Failed iterating table";
	if (ndb_rlIterate(&i,0) != l2) return "7: Failed iterating table";
	if (ndb_rlIterate(&i,0) != 0) return "8: Failed iterating table";
	
	//Now we switch the table of one data piece
	ndb_dataMk(db,l1,tbl);
	ndb_dataSetS(db,l1,l,2,"OHAI WORLD!");
	ndb_dataSetU16(db,l1,l,1,4353);
	ndb_dataSetU64(db,l1,l,0,43537965);
	ndb_rlAdd(db,l1,0);
	ndb_liIterinit(db,tmp,&i);
	
	if (ndb_rlIterate(&i,0) != l2) return "9: Failed iterating table";
	if (ndb_rlIterate(&i,0) != 0) return "10: Failed iterating table";
	
	tmp = ndb_rlGetli(db,0,tbl);
	if (!tmp) return "5: Getting has no data";
	
	ndb_liIterinit(db,tmp,&i);
	
	if (ndb_rlIterate(&i,tbl) != l1) return "9: Failed iterating table";
	if (ndb_rlIterate(&i,tbl) != 0) return "10: Failed iterating table";
	
	ndb_dataMk(db,l2,tbl);
	ndb_dataSetS(db,l2,l,2,"Shiiiiii!");
	ndb_dataSetU16(db,l2,l,1,2343);
	ndb_dataSetU64(db,l2,l,0,23434353);
	ndb_rlAdd(db,l2,0);
	
	if (ndb_rlIterate(&i,tbl) != l2) return "11: Failed iterating table";
	
	tmp = ndb_rlGetli(db,0,0);
	if (!tmp) return "5: Getting has no data";
	
	ndb_liIterinit(db,tmp,&i);
	
	if (ndb_rlIterate(&i,0) != 0) return "12: Failed iterating table";
	
	return NULL;
}
//-------------------------------------------------------------------
//Query
#include "nDB/query.h"

char* query(nDB* db) {
	return "Unimplemented";
	return NULL;
}

//-------------------------------------------------------------------
//Test framework

char test(char* system, char* result) {
	if (color) {
		printf("\033[01;33mTest: %-43s\t",system);
		if (!result) printf("[\e[1;32m+\033[01;33m]\033[22;37m\n");
		else {
			haderror=1;
			printf("[\e[1;31mFAILED\033[01;33m]\e[1;31m\nERROR %s\033[22;37m\n",result);
		}
	} else {
		printf("Test: %-40s\t",system);
		if (!result) printf("[+]\n");
		else {
			haderror=1;
			printf("[FAILED]\nERROR %s\n",result);
		}

	}
	return (result?1:0);
}

int main(int argc, char** argv) {
	int lvl = 0;
	int i;
	nDB* db;
	//Parse arguments
	for (i=1;i<argc;i++) {
		if (!strcmp(argv[i],"-l") || !strcmp(argv[i],"--log")) {
			i++;
			if (i>=argc || argv[i][0]=='-') {
				printf("l/log: Logs to a file, include a file name\n");
				return 1;
			} else {
				FILE* f = fopen(argv[i],"w");
				if (f){
					ndb_logSetFile(f);
					flogg=1;
				 } else {
				 	printf("l/log: Could not open file '%s'\n",argv[i]);
				 	return 1;
				 }
			}
		} else if (!strcmp(argv[i],"-c") || !strcmp(argv[i],"--color")) {
			color = 1;
		} else if (!strcmp(argv[i],"-o") || !strcmp(argv[i],"--level")) {
			i++;
			if (i>=argc || argv[i][0]=='-') {
				printf("o/level: Sets the level at which to log at\n");
				return 1;
			} else {
				lvl = atoi(argv[i]);
			}
		} else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")) {
			printf("nDB tests - these test the nDB library for problems\n");
			printf("Options:\n");
			printf("\t-l <filename>\t: logs to the file specified, rather than terminal\n");
			printf("\t-o\t\t: Sets the log level, default is 0\n");
#if defined(linux)
			printf("\t-c\t\t: Enables color-coding terminal output\n");
#endif
			return 1;
		} else {
			printf("Command '%s' not recognized\n",argv[i]);
			return 1;
		}
	}
#if defined(__GNUC__) && defined(linux)
	mtrace();	//Yaay for mtrace!
#endif
	if (color)
		printf("\033[01;33mRunning nDB tests\033[22;37m\n");
	else
		printf("Running nDB tests\n");
		
	ndb_logSetLevel(lvl);	//Sets level so that all messages are displayed
	//Here are the tests that are carried out
	
	/************************************************************************************
	
	The tests to be carried out
	
	************************************************************************************/
	
	test("Error Handling",error());
	if (!test("Mapping System",mapper())) {
		if (!test("Database Core",core())) {
			if (!test("Allocator",allocator())) {
				db = ndb_open(NULL);
				test("Offsets",offsets(db));
				test("Tables",tables(db));
				test("Data Storage",data(db));
				test("List Index",lindex(db));
				test("Tree Index",tindex(db));
				test("Root List",rlist(db));
				test("Database Queries",query(db));
				
				ndb_close(db);
			}
		}
	}
	
	/************************************************************************************
	
	
	
	************************************************************************************/
	
	
	
	if (flogg)
		fclose(ndb_logGetFile());
	
#if defined(__GNUC__) && defined(linux)
	//Yay for mtrace!
	muntrace();
#endif
//Pretty color coded text - only works in linux terminal
	if (color) {
		if (!haderror) {
			printf("\n\033[01;33mAll tests completed successfully.\e[0m\n");
		} else {
			printf("\n\e[1;31mThere was a FAILURE in one or more tests.\e[0m\n");
		}
	} else {
		//In the name of portability
		if (!haderror) {
			printf("\nAll tests completed successfully.\n");
		} else {
			printf("\nThere was a FAILURE in one or more tests.\n");
		}
	}

#ifdef _WIN32
	system("pause");
#endif
	return 0;
}

//EOF

