/**************************************************************************************

                         Allocation for Database Structures
                                    Definitions

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "allocator.h"
#include "offsets.h"	//For the endpoint of the database

#include <string.h>		//For memcpy in ralloc

NDB_LOC ndb_alloc(nDB* db, NDB_SIZE size) {
	NDB_LOC ptr = ndb_ohEnd(db);
	assert(db && size);
	ndb_log(NDB_LALC,"ALLOC: Allocating %u from database\n",size);
	
	if (db->size <= ptr + size) {
		//The database's current size is not large enough to fit the new allocation,
		//	so we need to expand the database itself
		db->resize((void*)db,size+NDB_RESIZE*db->size);
		
		if (db->size <= ptr + size) return 0;	//Unsuccessful at expanding mapping
	}
	
	//Set the endpoint after the next size
	ndb_ohEnd(db) = ptr + size;
	
	ndb_log(NDB_LTDO,"alloc: Assuming data at the end of the file is already null\n");
	
	return ptr;		//Return the 'allocated' pointer
}

void ndb_free(nDB* db, NDB_LOC loc, NDB_SIZE size) {
	assert(db && size && loc);
	ndb_log(NDB_LALC,"ALLOC: Freeing %u from location %lu\n",size,loc);
	ndb_log(NDB_LALC,"alloc: free is not implemented\n");
	ndb_log(NDB_LTDO,"alloc:TODO: implement trash\n");
}

NDB_LOC ndb_ralloc(nDB* db, NDB_LOC old, NDB_SIZE oldsize, NDB_SIZE newsize) {
	NDB_LOC ptr;

	assert(db && old && oldsize && newsize);
	assert(newsize > oldsize);
	
	ptr = ndb_alloc(db,newsize);	//Allocate a new region
	
	if (!ptr) return 0;			//Failed!
	
	//Copy data to new region
	memcpy(db->map+ptr,db->map+old,oldsize);
	
	//Free old region
	ndb_free(db,old,oldsize);
	
	ndb_log(NDB_LTDO,"alloc:TODO: make ralloc check for ability to resize");
	
	return ptr;
}

//EOF

