/**************************************************************************************

                                      HeapStruct
                               Methods to access Data

**************************************************************************************/
//Tab Width: 4 Character:\t

//Includes standard definitions
#include "ndb_def.h"

#define NDB_HEAP_RESIZE_OFFSET 3 //The additional space to allocate when resizing the heap

//-------------------------------------------------------------------------------------
//HEAPSTRUCT
//	This structure is like a memory allocator, it gives memory locations and specific sizes.
//	If it is told to write an amount larger than the allocated amount, it reallocates the area
//	and returns a new 'pointer'.
//TODO: Maybe some heap recycling? That could help cut down on the size of the heap, but admittedly
//		the database is not made to use tiny hard-drive space amounts.

NDB_FLOC ndb_mkHeap(nDB* db);	//Creates a heap

NDB_FLOC ndb_heapAlloc_(nDB* db,uint32_t size);	//Allocates size from the heap
#define ndb_heapAlloc ndb_heapAlloc_

char ndb_heapCopy_(nDB* db, NDB_FLOC to, NDB_FLOC from, uint32_t size, uint32_t offset);
#define ndb_heapCopy ndb_heapCopy_
	//Copies data from one heap location to another

NDB_FLOC ndb_heapWrite_(nDB* db, NDB_FLOC loc,void* object,uint32_t size,uint32_t offset);	
#define ndb_heapWrite ndb_heapWrite_
	//Writes the object to heap - if the size is greater than allocated amount, it reallocates
	//	to a new area and returns the location.
uint8_t ndb_heapRead_(nDB* db,NDB_FLOC loc,void* object, uint32_t size, uint32_t offset);
#define ndb_heapRead ndb_heapRead_
	//Reads an object from the heap - returns failure if the object is larger than the allocated size

char ndb_heapCmp(nDB* db, NDB_FLOC heaploc, uint32_t offset,void* object, uint32_t size);
	//Compares the object to another object on the heap

uint8_t ndb_heapStrcmp(nDB* db, NDB_FLOC hloc, char* string);
	//Does the equivalent of a string comparison
	
uint32_t ndb_heapSize(nDB* db, NDB_FLOC loc);
	//Returns the size of the object stored  in the heap


//EOF

