/**************************************************************************************

                                      Filesystem
                                    Implementation

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "filesystem.h"		//Includes self

//-------------------------------------------------------------------
//ANSI implementation of file IO
#ifndef NDB_NOANSI
#include <stdio.h>			//File management ANSI

void* ndb_ansiOpen(char* filename, uint8_t options) {
	if (options & NDB_FREAD) {				//If it has the read flag
		if (options & NDB_FWRITE)			//If it has the write flag
			return fopen(filename,"r+b");	//Open for read and write
		return fopen(filename,"rb");		//Open for read
	} else if (options & NDB_FWRITE) 		//If it has only the write flag
		return fopen(filename,"wb");		//Open for write
	return NULL;							//If no options, don't do anything
}

void ndb_ansiClose(void* file) {	fclose(file);	}

void ndb_ansiFlush(void* file) {	fflush(file);	}

int ndb_ansiRead(uint64_t loc, uint32_t numBytes,  void* readTo, void* file) {
	fseek(file,loc,SEEK_SET);
	return fread(readTo,1,numBytes,file);
}

int ndb_ansiWrite(uint64_t loc, uint32_t numBytes,  void* writeFrom, void* file) {
	fseek(file,loc,SEEK_SET);
	return fwrite(writeFrom,1,numBytes,file);
}

#endif	//End of ANSI C implementation


//-------------------------------------------------------------------
//Verbose Output Implementation

#if defined(NDB_FVER) && NDB_HASLOG	//Verbose versions of file access
#include <stdio.h>		//For printf

void* ndbver_ofile(char* filename, uint8_t options,char* fname,int lnum) {
	void* ptr = ndb_ofile_(filename,options);
	ndb_log(NDB_FVER,"\n(FOPEN  File:%-15lu| Opt:",(uint64_t)ptr);
	if (options & NDB_FREAD) {				//If it has the read flag
		if (options & NDB_FWRITE)			//If it has the write flag
			ndb_log(NDB_FVER,"RW");
		else ndb_log(NDB_FVER,"R ");
	} else if (options & NDB_FWRITE) 		//If it has only the write flag
		ndb_log(NDB_FVER,"W ");
	
	ndb_log(NDB_FVER," | FileName:%s | %s:%d )",filename,fname,lnum);
	return ptr;
}
void ndbver_cfile(void* file,char* fname,int lnum) {
	ndb_log(NDB_FVER,"\n(FCLOSE File:%-15lu| %s:%d )",(uint64_t)file,fname,lnum);
	ndb_cfile_(file);
}
int ndbver_wfile(uint64_t loc, uint32_t numbytes, void* writeTo, void* file,char* fname,int lnum) {
	ndb_log(NDB_FVER,"\n(FWRITE File:%-15lu| Ptr:%-8lu| Size:%-5u| %s:%d )",
		(uint64_t)file,loc,numbytes,fname,lnum);
	return ndb_wfile_(loc,numbytes,writeTo,file);
}
int ndbver_rfile(uint64_t loc, uint32_t numbytes, void* writeFrom, void* file,char* fname,int lnum) {
	ndb_log(NDB_FVER,"\n(FREAD  File:%-15lu| Ptr:%-8lu| Size:%-5u| %s:%d )",
		(uint64_t)file,loc,numbytes,fname,lnum);
	return ndb_rfile_(loc,numbytes,writeFrom,file);
}
void ndbver_ffile(void* file,char* fname,int lnum) {
	ndb_log(NDB_FVER,"\n(FWRITE File:%-15lu| %s:%d )",
		(uint64_t)file,fname,lnum);
	return ndb_ffile_(file);
}

#endif

//EOF

