/**************************************************************************************

                                 nDB Page-Level Access

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_PG
#	define NDB_PG

#include "ndb_def.h"	//Includes the nDB definitions
#include "pager.h"		//Includes the pager's access functions

//Defines the size of a page - allows reading after a page link
#define NDB_PGNSIZE	sizeof(NDB_PGNUM)

//Defines read page number to be read 64 bit unsigned int!
#define ndb_pgrpg ndb_pgru64
#define ndb_pgwpg ndb_pgwu64

/*
	NDB_PG is the header that all basic nDB structures use to access the database pages.
	It abstracts away getting new pages/Discarding old ones. It also takes care of the trash
	in the database, making sure to recycle pages as often as possible. As such, the trash
	is in itself a structure in the database. The location of the trash page is held in the 
	database object itself
*/

//Returns page locations and page number from a File Location
NDB_PGNUM ndb_locPage(nDB* db, NDB_FLOC p);
uint16_t ndb_locLoc(nDB* db, NDB_FLOC p);

//The difference between these two is that one checks a page
//	WITHOUT readlock, since you have the page, it assumes you
//	are writing more. The other, given just a page number automatically
//	locks the page
uint8_t ndb_checkLabelP(nDB* db, ndb_page* p, uint8_t lbl);
uint8_t ndb_checkLabel(nDB* db, NDB_PGNUM p, uint8_t lbl);

//Gets a page ONLY if the label checks out!
//	WARNING: Pages obtained with this have an automatic readlock!
//		Using it you have to manually call pg_Done on the page!
ndb_page* ndb_gcPage(nDB* db, NDB_PGNUM p, uint8_t lbl);

//Standard get new page/delete page functions
NDB_PGNUM ndb_newPage_(nDB* db);
void ndb_delPageP_(nDB* db,ndb_page* p);
#define ndb_newPage ndb_newPage_	//The definitions allow for logging to be enabled or disabled
#define ndb_delPageP ndb_delPageP_

//Two definitions to simplify getting/returning pages 
#define ndb_newPageP(a) ndb_getPage(a,ndb_newPage(a))
#define ndb_delPage(a,b) ndb_delPageP(a,ndb_getPage(a,b))	

//Trash structure actions
void ndb_delTrash_(nDB* db);	//Deletes the trash structure - effectively spilling trash pages
uint8_t ndb_mkTrash_(nDB* db);		//Creates a new trash structure
void ndb_pickupTrash_(nDB* db);	//Searches the database for free trash pages and adds them to the trash
#define ndb_delTrash ndb_delTrash_
#define ndb_mkTrash ndb_mkTrash_
#define ndb_pickupTrash ndb_pickupTrash_

NDB_PGNUM ndb_pgIterate(nDB* db, NDB_PGNUM loc);

//Iterate through all pages in the database which fit the description
NDB_PGNUM ndb_pgFind(nDB* db, NDB_PGNUM loc, uint16_t location, void* object, uint16_t size);

NDB_PGNUM ndb_pgFindChar(nDB* db, NDB_PGNUM loc, uint16_t location, uint8_t lbl);

//A simplification, since page labels are characters
#define ndb_pgFindLabel(a,b,c) ndb_pgFindChar(a,b,0,c)

//Enables Logging of actions
#if NDB_HASLOG && defined (NDB_NPGVER)

#undef ndb_newPage
#undef ndb_delPageP

#define ndb_newPage ndb_newPage_l
#define ndb_delPageP ndb_delPageP_l

#define ndb_newPage_l(a) ndb_newPage__(a,__FILE__,__LINE__)
#define ndb_delPageP_l(a,b) ndb_delPageP__(a,b,__FILE__,__LINE__)

NDB_PGNUM ndb_newPage__(nDB* db, char* fname, int lnum);
void ndb_delPageP__(nDB* db, ndb_page* p, char* fname, int lnum);

#endif

#if NDB_HASLOG && defined (NDB_TVER)

#undef ndb_delTrash
#undef ndb_mkTrash
#undef ndb_pickupTrash

#define ndb_delTrash ndb_delTrash_l
#define ndb_mkTrash ndb_mkTrash_l
#define ndb_pickupTrash ndb_pickupTrash_l

#define ndb_delTrash_l(a) ndb_delTrash__(a,__FILE__,__LINE__)
#define ndb_mkTrash_l(a) ndb_mkTrash__(a,__FILE__,__LINE__)
#define ndb_pickupTrash_l(a) ndb_pickupTrash__(a,__FILE__,__LINE__)

void ndb_delTrash__(nDB* db, char* fname, int lnum);
uint8_t ndb_mkTrash__(nDB* db, char* fname, int lnum);
void ndb_pickupTrash__(nDB* db, char* fname, int lnum);

#endif

#endif

//EOF

