/**************************************************************************************

                                   nDB Basic Tests

**************************************************************************************/
//Tab Width: 4 Character:\t
#include <stdio.h>
#include <string.h>

#if defined(__GNUC__) && defined(linux)
#include <mcheck.h>
#endif

//Global, did everything go correctly
char haderror = 0;

//-------------------------------------------------------------------
//Test of Error handler
#include "nDB/error.h"

char error() {
	nDB a;
	ndb_errinit(&a);	//Supposed to reset 
	if (ndb_peekerr(&a)!= 0) return 1;
	ndb_seterr(&a,3);
	if (ndb_peekerr(&a) != 3) return 2;	//It is supposed to set that
	if (ndb_peekerr(&a) != 3) return 3;	//It is supposed to keep it
	if (ndb_geterr(&a)!=3) return 4;//Return the error number!
	if (ndb_peekerr(&a) != 0) return 5;	//It is supposed to keep it
	
	//Now just check if it returns a pointer from getstring
	if (!ndb_geterrStr(&a)) return 6;
	//This actually does nothing
	ndb_errclose(&a);
	return 0;
}

//-------------------------------------------------------------------
//Test of file IO
#include "nDB/filesystem.h"

char filesystem() {
	void* f = ndb_ofile("ndb_testfile.test",NDB_FWRITE);
	uint32_t a = 30;
	uint64_t b = 60;
	uint8_t c = 90;
	
	ndb_wfile(0,sizeof(a),&a,f);
	ndb_wfile(90,sizeof(b),&b,f);
	ndb_wfile(30,sizeof(c),&c,f);
	
	ndb_ffile(f);
	
	ndb_cfile(f);
	
	f = ndb_ofile("ndb_testfile.test",NDB_FREAD | NDB_FWRITE);
	
	a=0;b=0;c=0;
	ndb_rfile(90,sizeof(b),&b,f);
	ndb_rfile(0,sizeof(a),&a,f);
	ndb_rfile(30,sizeof(c),&c,f);
	
	ndb_cfile(f);
	
	if (remove("ndb_testfile.test") == -1)
		return 4;
	
	if (a != 30) return 1;
	if (b != 60) return 2;
	if (c != 90) return 3;
	
	return 0;
}

//-------------------------------------------------------------------
//Test of Cache Functions
#include "nDB/cache.h"

char cache_checkPage(ndb_page* p) {
	//Make sure the page is up to standards
	//TODO: It does not yet check if the data is 0'd.
	if (!p) return -1;
	if (p->modified) return 1;
	if (p->page) return 2;
	if (p->writing) return 3;
	if (!p->data) return 4;
	return 0;
}

char cache() {
	//NOTE: This is NOT how to create a database. This is just a test
	//	to make sure it works well internally
	nDB db;
	ndb_page* p1;
	ndb_page* p2;
	ndb_page* p3;
	ndb_cacheIter i=0;
	int i2 = 0;
	
	memset(&db,0,sizeof(nDB));	//Set the database to 0;
	db.pagesize		=10;
	db.pagenum		=1;
	db.ppool_size	=2;
	db.cache_addon	=1;
	
	//Initialize the cache with the given parameters
	if (ndb_cacheInit(&db)) return -1;
	
	
	//First, get the pages from the ppool. Notice
	//	that we get one more than the ppool array
	//	to test refill
	p1 = ndb_cacheNewPage(&db);
	p2 = ndb_cacheNewPage(&db);
	p3 = ndb_cacheNewPage(&db);
	
	if (cache_checkPage(p1)	//Make sure it is initialized properly
		||	cache_checkPage(p2)
		||	cache_checkPage(p3)) return 1;
	
	
	if (ndb_pgInUse(p1) || ndb_pgInUse(p2) || ndb_pgInUse(p3)) return 2;
	if (ndb_pgReaders(p1) || ndb_pgReaders(p2) || ndb_pgReaders(p3)) return 3;
	if (ndb_pgIsLocked(p1) || ndb_pgIsLocked(p2) || ndb_pgIsLocked(p3)) return 4;
	if (ndb_pgIsModified(p1) || ndb_pgIsModified(p2) || ndb_pgIsModified(p3)) return 5;
	
	//Now we test the locking mechanisms
	
	//Set the write lock
	ndb_pgWriteLock(p1);
	
	if (!ndb_pgInUse(p1) || !ndb_pgIsLocked(p1)) return 6;
	
	p1->page = 6;	//Out of bounds of the current array! It will have to reallocate
	p1->data[5]=1;	//Set some data!
	
	ndb_pgWriteUnlock(p1);
	if (!ndb_pgIsModified(p1)) return 7;	//the page should be modified
	
	p2->page = 1;
	p2->data[1]=1;
	
	p3->data[2]=1;
	p3->page = 0;
	
	ndb_pgRead(p2);
	//There is one reader
	if (ndb_pgReaders(p2) != 1 || !ndb_pgInUse(p2)) return 8;
	ndb_pgDone(p2);
	//There are no readers
	if (ndb_pgReaders(p2) != 0 || ndb_pgInUse(p2)) return 9;
	//It is not modified
	if (ndb_pgIsModified(p2)) return 10;
	
	ndb_cacheAddPage(&db,p1);
	ndb_cacheAddPage(&db,p2);
	ndb_cacheAddPage(&db,p3);
	
	while ((p1 = ndb_cacheIterate(&db,&i))) {
		i2++;
		if (!p1 || (!p1->data[5] && !p1->data[1] && !p1->data[2]) || p1->data[0])
			return 11;	//Make sure we get the right pages
	}
	if (i2 != 3) return 12;	//There are 3 test pages
	
	if (ndb_cacheGet(&db,6)->data[5]!=1) return 13;
	if (ndb_cacheGet(&db,0)->data[2]!=1) return 14;
	if (ndb_cacheGet(&db,1)->data[1]!=1) return 15;
	if (ndb_cacheGet(&db,3) != NULL) return 16;	//This page DNE
	
	//One page is modified, none are in use.
	ndb_cacheClean(&db);
	
	if (ndb_cacheGet(&db,6)->data[5]!=1) return 17;
	if (ndb_cacheGet(&db,0)!=NULL) return 18;
	if (ndb_cacheGet(&db,1)!=NULL) return 19;
	if (ndb_cacheGet(&db,50) != NULL) return 20;	//This page DNE

	p1 = ndb_cacheNewPage(&db);
	p2 = ndb_cacheNewPage(&db);
	p3 = ndb_cacheNewPage(&db);
	
	if (cache_checkPage(p1)	//Make sure it is initialized properly
		||	cache_checkPage(p2)
		||	cache_checkPage(p3)) return 21;
		
	ndb_cacheDelPage(&db,p2);	//Check page deleting
	
	p1->page=3;p3->page=1;
	ndb_cacheAddPage(&db,p1);
	ndb_cacheAddPage(&db,p3);
	if (ndb_cacheGet(&db,3)!=p1) return 22;
	//Removes the page from the cache
	ndb_cacheRemovePage(&db,p1);
	if (ndb_cacheGet(&db,3)!=NULL) return 23;
	
	//Close should free all pages left
	ndb_cacheClose(&db);
	
	return 0;
}

//-------------------------------------------------------------------
//Test of Pager Functions
#include "nDB/pager.h"
char pager() {
	nDB db;
	ndb_page* p1;
	ndb_page* p2;
	ndb_page* p3;
	
	ndb_setLogLevel(2);	//Show higher level logging only
	
	memset(&db,0,sizeof(nDB));	//Set the database to 0;
	db.pagesize		=30;
	db.pagenum		=1;
	db.ppool_size	=4;
	db.cache_addon	=0;			//Test different cache settings
	
	ndb_pagerInit(&db);
	
	//Obtain pages
	p1= ndb_getPage(&db,0);
	p2= ndb_getPage(&db,2);
	p3= ndb_getPage(&db,2);
	if (p2!=p3) return 1;
	if (p3->users!=2) return 2;
	//Free page
	ndb_freePage(p3);
	
	//There is supposed to be only one user
	if (p2->users!=1) return 3;
	
	ndb_pgSetWriteLock(p1);
	
	//Write some stuff
	ndb_pgwu16(p1,0,543);
	ndb_pgwu32(p1,8,94282);
	ndb_pgwu64(p1,17,43545351);
	
	ndb_pgputc(p1,7,'L');
	
	if (!ndb_pgIsLocked(p1)) return 4;
	
	ndb_pgWriteUnlock(p1);
	
	if (ndb_pgIsLocked(p1)) return 5;
	
	ndb_pgRead(p1);
	if (ndb_pgru32(p1,8)!=94282) return 6;
	if (ndb_pgru16(p1,0)!=543) return 7;
	if (ndb_pgru64(p1,17)!=43545351) return 8;
	if (ndb_pgru32(p1,8)!=94282) return 9;
	if (ndb_pggetc(p1,7)!='L') return 10;
	ndb_pgDone(p1);
	
	ndb_pgWriteLock(p1);
	ndb_pgclear(&db,p1);
	ndb_pgWriteUnlock(p1);
	
	ndb_pgRead(p1);
	if (ndb_pgru32(p1,8)!=0) return 11;
	if (ndb_pgru16(p1,0)!=0) return 12;
	if (ndb_pgru64(p1,17)!=0) return 13;
	if (ndb_pgru32(p1,8)!=0) return 14;
	if (ndb_pggetc(p1,7)!=0) return 15;
	ndb_pgDone(p1);
	
	ndb_pgWriteLock(p1);
	*((uint64_t*)ndb_pgptr(p1,5)) = 435324523;
	ndb_pgWriteUnlock(p1);
	
	ndb_pgRead(p1);
	if (ndb_pgru64(p1,5)!=435324523) return 16;
	ndb_pgDone(p1);
	
	ndb_freePage(p1);
	
	ndb_pgWriteLock(p2);
	ndb_pgstrw(p2,4,"Hello World!");
	ndb_pgWriteUnlock(p2);
	
	ndb_pgRead(p2);
	if (ndb_pgstrlen(p2,4)!=12) return 17;
	ndb_pgDone(p2);
	
	p3 = ndb_getPage(&db,8);
	
	//We don't do a write lock, to make sure cleaning works correctly
	ndb_pgputc(p3,3,'O');
	
	ndb_freePage(p3);
	
	ndb_cacheClean(&db);
	
	p3 = ndb_getPage(&db,8);
	if (ndb_pggetc(p3,3)!=0) return 18;	//The page should have been dumped
	
	if (strcmp(ndb_pgptr(p2,4),"Hello World!")) return 19;
	
	p3=ndb_getPage(&db,0);
	if (ndb_pgru64(p3,5)!=435324523) return 20;
	
	//Now to check the extra functions (added during the making of trash)
	
	ndb_putlP(p3,'P');						//Writes the label
	if ('P'!=ndb_pggetc(p3,0)) return 21;
	
	//Now we check the pgCompare
	{
		uint64_t z = 435324523;
		if (ndb_pgCompare(p3,5,&z,sizeof(uint64_t))) return 22;
		--z;
		if (!ndb_pgCompare(p3,5,&z,sizeof(uint64_t))) return 23;
	}
	
	ndb_pagerClose(&db);
	
	//Looks like everythin is working fine
	return 0;
}

//-------------------------------------------------------------------
//Test of Trash paging

#include "nDB/ndb_pg.h"

char trash() {
	//Okay, let's first initialize one without a trash
	nDB db;
	NDB_PGNUM pg;
	ndb_page* p1;
	ndb_page* p2;
	//ndb_page* p3;
	ndb_setLogLevel(4);	//Show higher level logging only
	
	memset(&db,0,sizeof(nDB));	//Set the database to 0;
	db.pagesize		=40;
	db.pagenum		=1;
	db.ppool_size	=1;
	db.cache_addon	=4;			//Test different cache settings
	
	ndb_pagerInit(&db);
	
	//Okay, so what to do first? trash is NULL
	if (db.trash) return 1;
	
	pg=ndb_newPage(&db);
	if (pg!=1) return 2;
	pg = ndb_newPage(&db);
	if (pg!=2) return 3;
	ndb_delPage(&db,1);
	pg=ndb_newPage(&db);
	if (pg!=3) return 4;	//It has no trash to put it in!
	ndb_mkTrash(&db);		//Trash takes page 4
	ndb_delPage(&db,2);
	ndb_delPage(&db,3);		//There are 2 pages in the trash, it has one page
	pg = ndb_newPage(&db);
	if (pg!=3) return 5;
	pg = ndb_newPage(&db);
	if (pg!=2) return 6;	//There are no pages in the trash
	pg = ndb_newPage(&db);
	if (pg!=5) return 7;	//It gives back even another page, since it ran out of 
							//	trash pages
	if (db.pagenum != 6) return 8;
	if ((6 != ndb_newPage(&db))) return 9;
	if ((7 != ndb_newPage(&db))) return 10;
	if ((8 != ndb_newPage(&db))) return 11;
	if ((9 != ndb_newPage(&db))) return 12;
	//Delete the current pages: 4*8=>32+8+2+1=>43
	ndb_delPage(&db,2);
	ndb_delPage(&db,3);					
	ndb_delPage(&db,1);					//These three are already used
	ndb_delPage(&db,5);					//5)Trash
	ndb_delPage(&db,6);	//6
	ndb_delPage(&db,7);	//7
	ndb_delPage(&db,8);	//8
	ndb_delPage(&db,9);	//9)Trash
	
	if (ndb_newPage(&db)!=9) return 13;
	if (ndb_newPage(&db)!=8) return 14;
	if (ndb_newPage(&db)!=7) return 15;
	if (ndb_newPage(&db)!=6) return 16;
	if (ndb_newPage(&db)!=5) return 17;
	if (ndb_newPage(&db)!=1) return 18;
	if (ndb_newPage(&db)!=3) return 19;
	if (ndb_newPage(&db)!=2) return 20;
	if (ndb_newPage(&db)!=10) return 21;
	
	//Now we write a number to two pages
	{
	uint32_t num = 23423424;
	NDB_PGNUM iter = 0;
	int i=0;
	
	p1 = ndb_getPage(&db,6);
	p2 = ndb_getPage(&db,9);
	ndb_pgwu32(p1,9,num);
	ndb_pgwu32(p2,9,num);
	//ndb_freePage(p1); // Used later
	//ndb_freePage(p2);
	
	while ((iter = ndb_pgFind(&db,iter,9,&num,sizeof(uint32_t)))) {
		i++;
		if (iter != 6 && iter != 9) return 22;
	}
	if (i!=2) return 23;
	}
	//Iterator was now tested. All pages start with the free label, so deleting and
	//	refilling trash should give me exactly that.
	
	//Label page 6 with non-free label
	ndb_putlP(p1,'W');
	ndb_putlP(p2,'W');	//Checks overwriting as free
	
	//Throw away some pages, so that we can test trash deletion
	ndb_delPage(&db,2);
	ndb_delPage(&db,3);					
	ndb_delPage(&db,1);
	ndb_delPage(&db,5);		//5)Trash
	ndb_delPage(&db,7);
	ndb_delPage(&db,6);
	
	ndb_delTrash(&db);	//Should free everything correctly
	
	if (ndb_newPage(&db)!=11) return 24;
	ndb_pickupTrash(&db);	//Shouls pick up all the pages so far except 9.
	
	if (ndb_newPage(&db)!=11) return 25;
	if (ndb_newPage(&db)!=10) return 26;
	if (ndb_newPage(&db)!=8) return 27;
	if (ndb_newPage(&db)!=7) return 28;
	if (ndb_newPage(&db)!=6) return 29;
	
	ndb_delTrash(&db);
	ndb_pagerClose(&db);
	
	
	return 0;
}




//-------------------------------------------------------------------
//Test framework

void test(char* system, char result) {
#if defined(__GNUC__) && defined(linux) && defined(TEST_COLOR)
	printf("\n\033[01;33mTest: %-40s\t",system);
	if (!result) printf("[\e[1;32m+\033[01;33m]\033[22;37m");
	else {
		haderror=1;
		printf("[\e[1;31mFAILED @%i\033[01;33m]\033[22;37m",result);
	}
#else
	printf("\nTest: %-40s\t",system);
	if (!result) printf("[+]");
	else {
		haderror=1;
		printf("[FAILED @%i]",result);
	}

#endif
}

int main() {
#if defined(__GNUC__) && defined(linux)
	mtrace();	//Yaay for mtrace!
#endif
#if defined(__GNUC__) && defined(linux) && defined(TEST_COLOR)
	printf("\033[01;33mRunning nDB tests\033[22;37m");
#else
	printf("Running nDB tests");
#endif
	//ndb_setLogFile(fopen("loggy.txt","w"));
	
	//Here are the tests that are carried out	
	test("Error Handling",error());
	test("File IO",filesystem());
	test("Cache",cache());
	test("Pager",pager());
	test("Trash/PageAccess:",trash());
	
	//fclose(ndb_getLogFile());
	
#if defined(__GNUC__) && defined(linux)
	//Yay for mtrace!
	muntrace();
#endif
#if defined(__GNUC__) && defined(linux) && defined(TEST_COLOR)
//Pretty color coded text
	if (!haderror) {
		printf("\n\n\033[01;33mAll tests completed successfully.\e[0m\n");
	} else {
		printf("\n\n\e[1;31mThere was a FAILURE in one or more tests.\e[0m\n");
	}
#else
	//In the name of portability
	if (!haderror) {
		printf("\n\nAll tests completed successfully.\n");
	} else {
		printf("\n\nThere was a FAILURE in one or more tests.\n");
	}
#endif
	return 0;
}

//EOF

