/**************************************************************************************

                         Allocation for Database Structures
                                 Implementation

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "allocator.h"	//Includes self
#include "header.h"		//We need to obtain data from the header

#include <string.h>		//For memcpy in ralloc
//-------------------------------------------------------------------------------------

NDB_LOC ndb_alloc_(nDB* db, NDB_SIZE size) {
	NDB_LOC ptr = ndb_hgetEnd(db);
	
	assert(db && size && db->size);	//These have to be defined
	
	ndb_log(NDB_LTODO,"alloc: Assuming data at the end of the file is already null");
	
	if (db->size <= ptr +size) {
		//The database current size is not large enough to fit the new allocation,
		//	so we are forced to expand the database itself
		db->resize((void*)db,NDB_RESIZE*size+db->size);
		
		if (db->size <= ptr+size) {
			ndb_errSetdb(db,NDB_EMEM);	//Set memory error
			return 0;					//Unsuccessful at adding more allocationspace
		}
	}
	
	//Set the endpoint after the next size
	ndb_hsetEnd(db,ptr+size);
	
	//Return the 'allocated' pointer
	return ptr;
}
void ndb_free_(nDB* db, NDB_LOC loc, NDB_SIZE size) {
	assert(db && loc && size);	//These must exist
	
	ndb_log(NDB_LALC,"free: not implemented");
	ndb_log(NDB_LTODO,"free:TODO: implement trash");
	
}
NDB_LOC ndb_ralloc_(nDB* db, NDB_LOC old, NDB_SIZE oldsize, NDB_SIZE newsize) {
	NDB_LOC ptr;	//A veriable
	
	assert(db && old && oldsize && newsize && newsize > oldsize);
	
	//Allocate new region
	ptr = ndb_alloc_(db,newsize);
	
	if (!ptr) return 0;
	
	//Copy data to new region
	memcpy(db->map+ptr,db->map+old,oldsize);
	
	//Free old region
	ndb_free_(db,old,oldsize);
	
	ndb_log(NDB_LTODO,"ralloc:TODO: make ralloc check for ability to resize");
	return ptr;
}

#if NDB_LOG

NDB_LOC ndb_alloc__(nDB* db, NDB_SIZE size,NDB_LF) {
	ndb_ilog(NDB_LALC,"%8.8s:%-4d| ALLOC: Allocating %lu from database\n",f,l,size);
	return ndb_alloc_(db,size);
}

void ndb_free__(nDB* db, NDB_LOC loc, NDB_SIZE size,NDB_LF) {
	ndb_ilog(NDB_LALC,"%8.8s:%-4d| FREE: Freeing %lu from location %lu\n",f,l,size,loc);
	ndb_free_(db,loc,size);
}

NDB_LOC ndb_ralloc__(nDB* db, NDB_LOC old, NDB_SIZE oldsize, NDB_SIZE newsize,NDB_LF) {
	ndb_ilog(NDB_LALC,"%8.8s:%-4d| RALLOC: Reallocating from %lu (%lu->%lu)\n",f,l,old,oldsize,newsize);
	return ndb_ralloc_(db,old,oldsize,newsize);
}

#endif

//EOF

