/**************************************************************************************

                                   Database Core
                                  Implementation

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "base.h"
#include "header.h"		//Gets the end size
#include "filesystem.h"	//Get the memory manipulation stuff

#include <stdio.h>	//To be able to remove file
#include <string.h>	//memcpy

//-------------------------------------------------------------------------------------

nDB* ndb_open(char* file,uint8_t options) {
	nDB* db = malloc(sizeof(nDB));	//Allocates memory for the database
	
	if (!ndb_open_v2(db,file,options)) return db;
	
	//Failed, so return
	free(db);
	return NULL;
}

char ndb_open_v2_(nDB* db, char* file,uint8_t options) {
	assert(db);	//There has to be a database, no?
	
	memset(db,0,sizeof(nDB));	//Clears the database
	
	db->size = NDB_RESIZE * 4096;	//Sets the starting database size
	
	//This option only has an effect on file databases
	if (options & NDB_READONLY) db->readonly = 1;
	
	if (file) {
		//File-based database
		
		char isdb = ndb_checkFile(file);
		
		if (isdb <= 0) {
			//If we can read it
			uint8_t opt = (db->readonly?NDB_FR:NDB_FRW);
		
			if (!ndb_mapFile(db,file,opt)) {
		
				//The file is mapped! We now set some shit
				db->resize = ndb_mapFileResize;
				db->close = ndb_mapFileClose;
			
				if (isdb == -1) {
					//The file did not exist yet - write a header
					ndb_hwrite(db);
				}
				return 0;
			}
		}
	} else {
		//In-memory database bitches!
		
		if (!ndb_mapMem(db,db->size)) {
		
			//The file is mapped! We now set some shit
			db->resize = ndb_mapMemResize;
			db->close = ndb_mapMemClose;
			
			//Write the header
			ndb_hwrite(db);
			
			return 0;
		}
	}
	ndb_log(NDB_LMSG,"open: Failed to open database");
	return -1;
}

//Close database
void ndb_close(nDB* db) {	
	ndb_close_v2(db);
	free(db);
}

void ndb_close_v2_(nDB* db) {
	assert(db);	//A database has to exist, no?
	
	db->close((void*)db);	//Closes the memory of the database
}

//Creates an identical copy of the database either to file or to memory
nDB* ndb_copy_(nDB* db, char* filename) {
	nDB* newdb;
	
	assert(db);	//The old database has to exist, no?
	
	//WARNING: This totally wipes the file it does this to
	
	newdb = ndb_open(filename,0);	//Creates a new database
	
	if (newdb) {
		if (db->size > newdb->size)
			newdb->resize(newdb,db->size);	//resizes the new database
		if (newdb->size == db->size) {
			
			//The sizes match, let's get it on!
			ndb_log(NDB_LBAS,"copy: Copying database contents...");
			memcpy(newdb->map,db->map,ndb_hgetEnd(db));
			ndb_log(NDB_LBAS,"copy: Done");
			//Aww yeah
			return newdb;
			
		}
		ndb_close(newdb);
		
		if (filename) {			//If it fails, remove the file we just created
			remove(filename);	//This might be dangerous
			ndb_log(NDB_LMSG,"copy: Removing file");
		}
	}
	ndb_log(NDB_LMSG,"copy: failed");
	return NULL;
}

//-------------------------------------------------------------------------------------
//Logging
#if NDB_LOG
char ndb_open_v2__(nDB* db, char* file,uint8_t options,NDB_LF) {
	if (file) ndb_ilog(NDB_LBAS,"%8.8s:%-4d| OPEN: Opening database '%s'\n",f,l,file);
	else ndb_ilog(NDB_LBAS,"%8.8s:%-4d| OPEN: Creating in-memory database\n",f,l);
	return ndb_open_v2_(db,file,options);
}

void ndb_close_v2__(nDB* db,NDB_LF) {
	ndb_ilog(NDB_LBAS,"%8.8s:%-4d| CLOSE: Closing database\n",f,l);
	ndb_close_v2_(db);
}

nDB* ndb_copy__(nDB* db, char* filename,NDB_LF) {
	if (filename) ndb_ilog(NDB_LBAS,"%8.8s:%-4d| COPY: Copying database to '%s'\n",f,l,filename);
	else ndb_ilog(NDB_LBAS,"%8.8s:%-4d| COPY: Copying database\n",f,l);
	return ndb_copy_(db,filename);
}

#endif

//EOF

