/**************************************************************************************

                             Error Handling & Logging
                                    Header

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_ERRLOG
#	define NDB_ERRLOG

#include "ndb_def.h"	//The standard definitions

//-------------------------------------------------------------------------------------
//Logging:

#if NDB_LOG
//If the log exists

#include <stdio.h>

//Simplifies the logfunction writing
#define NDB_LD	__FILE__,__LINE__
#define NDB_LF	char* f,uint32_t l

FILE* ndb_logGetFile();							//Returns the logfile
void ndb_logSetFile(FILE* f);					//Sets the logfile (Actually a file pointer)
uint8_t ndb_logGetLevel();						//Gets the log level
void ndb_logSetLevel_(uint8_t level,NDB_LF);			//Sets the log level
void ndb_ilog(uint8_t level,char* str, ...);	//The internal logging function

	//Logs the given text to file
void ndb_log_(NDB_LF,uint8_t level, char* str, ...);	//The notification function
#define ndb_log(...) ndb_log_(NDB_LD,__VA_ARGS__)
#define ndb_logSetLevel(a) ndb_logSetLevel_(a,NDB_LD);
#else

//The log does not exist
#define ndb_logGetFile() NULL
#define ndb_logSetFile(a) 
#define ndb_logSetLevel(a) 
#define ndb_ilog(...) 
#define ndb_log(...)

#endif

//-------------------------------------------------------------------------------------
//Error Management:


#define NDB_EFILE 2		//File error
#define NDB_EMEM 1		//Memory error

void ndb_errSetdb_(nDB* db, uint8_t e);		//Sets a database error
void ndb_errSet_(nDB* db);					//Sets a system/unknown error
void ndb_errSetcustom_(nDB* db, char* err);	//Sets a user-defined string as error

char* ndb_errPeek(nDB* db);		//Returns the current error without clearing it
char* ndb_errGet(nDB* db);		//Returns the current error, resetting it
uint8_t ndb_errGetdb(nDB* db);	//Returns the database error code, if any

#if NDB_LOG
//There is a log: The functions are to be overloaded

#define ndb_errSetdb(a,b) ndb_errSetdb__(a,b,NDB_LD)
#define ndb_errSet(a) ndb_errSet__(a,NDB_LD)
#define ndb_errSetcustom(a,b) ndb_errSetcustom__(a,b,NDB_LD)

void ndb_errSetdb__(nDB* db, uint8_t e,NDB_LF);
void ndb_errSet__(nDB* db,NDB_LF);
void ndb_errSetcustom__(nDB* db,char* err,NDB_LF);

#else
//There is no log, the functions are to be defined

#define ndb_errSetdb ndb_errSetdb_
#define ndb_errSet ndb_errSet_
#define ndb_errSetcustom ndb_errSetcustom_

#endif

#endif
//EOF

