/**************************************************************************************

                                   nDB Basic Tests

**************************************************************************************/
//Tab Width: 4 Character:\t
#include <stdio.h>
#include <string.h>

#if defined(__GNUC__) && defined(linux)
#include <mcheck.h>
#endif

//Global, did everything go correctly
char haderror = 0;
char flogg = 0;
char color = 0;

//-------------------------------------------------------------------
//Test of Error handler
#include "nDB/log_err.h"

char* error() {
	nDB a;
	a.err=0;
	
	if (ndb_errPeek(&a)!=0) return "Error was initially non-NULL";
	ndb_errSetdb(&a,1);
	if (ndb_errGetdb(&a) != 1) return "Error Set failed to accomplish";
	if (ndb_errGetdb(&a) != 1) return "Error value was not saved";
	fopen("FileDnE.sfd.sdfdsf","r");
	ndb_errSet(&a);
	return NULL;
}

//-------------------------------------------------------------------
//Test of Filesystem
#include "nDB/filesystem.h"

char* filesystem() {
	//We create a database
	nDB a;
	void *trollface;
	a.map = NULL;	//NULL the associated parts
	a.file= 0;
	a.size= 0;
	
	//First try a memory database
	ndb_mapMem(&a,100);
	if (!a.map || a.size<100) return "1: Failed to create a map of the right size";
	if (*((uint64_t*)(a.map+90))) return "2: Map was initially not empty!";
	*((uint64_t*)(a.map+90)) = 865744534;
	if (*((uint64_t*)(a.map+90)) != 865744534) return "3: Map writing failed!";
	
	trollface = malloc(3);	//Allocate memory right after it (hopefully) so that it is forced to copy
	
	ndb_mapMemResize(&a,150);
	
	free(trollface);
	
	if (!a.map || a.size<150) return "4: Resizing map failed";
	if (*((uint64_t*)(a.map+90)) != 865744534) return "5: Data was not copied to new map";
	if (*((uint64_t*)(a.map+120))!=0) return "6:New map locations are non-NULL";
	
	ndb_mapMemClose(&a);
	
	//Now we map a REAL file
	a.size = 50;
	if (ndb_mapFile(&a,"test.db",NDB_FRW)) return "7: File mapping failed";
	if (!a.map || a.size <50) return "8: Mapping does not set values";
	
	if (*((uint64_t*)(a.map+30))) return "9: Map was initially not empty!";
	*((uint64_t*)(a.map+30)) = 865744534;
	if (*((uint64_t*)(a.map+30)) != 865744534) return "10: Map writing failed!";
	if (ndb_mapFileResize(&a,15023)) return "11: Map resize failed";
	if (!a.map || a.size<5023) return "12: Resizing map failed at values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "13: Data was not copied to new map";
	if (*((uint64_t*)(a.map+14020))!=0) return "14:New map locations are non-NULL";
	*((uint64_t*)(a.map+14900)) = 865744034;
	ndb_mapFileClose(&a);
	
	if (ndb_mapFile(&a,"test.db",NDB_FRW)) return "15: File mapping failed";
	if (!a.map || a.size <15023) return "16: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "17: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "18: Map read failed!";
	ndb_mapFileClose(&a);
	
	
	//Now that RW was tested, we can try out readonly access
	if (ndb_mapFile(&a,"test.db",NDB_FR)) return "19: File mapping failed";
	if (!a.map || a.size <15023) return "20: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "21: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "22: Map read failed!";
#ifndef _WIN32	//On windows, you can't do writing to readonly mappings, so this fails
	*((uint64_t*)(a.map+150)) = 4354;
	if (*((uint64_t*)(a.map+150)) != 4354) return "23: Map read failed!";
	/*Currently, mapFileResize can't resize read-only files, so this part is t3h fale
	if (ndb_mapFileResize(&a,50023)) return "24: Map resize failed";
	*((uint64_t*)(a.map+49000)) = 4355;
	if (*((uint64_t*)(a.map+150)) != 4354) return "25: Map read failed!";
	if (*((uint64_t*)(a.map+49000)) != 4355) return "26: Map read failed!";
	*/
	ndb_mapFileClose(&a);
	
	//Now make sure that it was NOT written to file!
	if (ndb_mapFile(&a,"test.db",NDB_FR)) return "27: File mapping failed";
	if (!a.map || a.size <15023) return "28: Mapping does not set values";
	if (*((uint64_t*)(a.map+30)) != 865744534) return "29: Map read failed!";
	if (*((uint64_t*)(a.map+14900)) != 865744034) return "30: Map read failed!";
	if (*((uint64_t*)(a.map+150)) != 0) return "31: Map read failed!";
	if (a.size >= 40000) return "32: Map file sizing incorrect!";
#endif
	ndb_mapFileClose(&a);
	
	remove("test.db");
	return 0;
}

//-------------------------------------------------------------------
//Header Access

#include "nDB/header.h"
char* header() {
	nDB a;

	//First we create 3 files - one with a failed id, one with a large version
	//	and one that is good - and we make sure that checkFile works correctly
	FILE* f = fopen("nDB.test.1.db","w");
	fwrite("nqB",4,1,f);
	putc(1,f);
	fclose(f);
	f = fopen("nDB.test.2.db","w");
	fwrite("nDB",4,1,f);
	putc(255,f);
	fclose(f);
	f = fopen("nDB.test.3.db","w");
	fwrite("nDB",4,1,f);
	putc(1,f);
	fclose(f);
	
	//Check the files
	if (!ndb_checkFile("nDB.test.1.db")) return "1: Checking database validity failed";
	if (!ndb_checkFile("nDB.test.2.db")) return "2: Checking database validity failed";
	if (ndb_checkFile("nDB.test.3.db")) return "3: Checking database validity failed";
	
	//Clear out the files
	remove("nDB.test.1.db");
	remove("nDB.test.2.db");
	remove("nDB.test.3.db");
	
	//So checkfile works
	
	//Now we check makeHeader
	a.map = malloc(50);
	a.size=50;
	
	ndb_hwrite(&a);	//Writes the header
	
	if (strcmp(a.map,"nDB")) return "4: Header Creation Failed";
	if (a.map[4]!=NDB_VER) return "5: Header version mismatch";
	
	//Now we set each thing individually, checking to make sure they don't f us up
	if (ndb_hgetTables(&a)) return "6: Database variables not null";
	if (ndb_hgetNodes(&a)) return "7: Database variables not null";
	if (ndb_hgetTrash(&a)) return "8: Database variables not null";
	
	//The end variable is already set
	
	//Sets the values
	ndb_hsetTables(&a,32434235);
	ndb_hsetNodes(&a,32434234);
	ndb_hsetTrash(&a,32434233);
	ndb_hsetEnd(&a,32434232);
	
	//Checks to make sure all is well
	if (32434235 != ndb_hgetTables(&a)) return "9: Database variables not set";
	if (32434234 != ndb_hgetNodes(&a)) return "10: Database variables not set";
	if (32434233 != ndb_hgetTrash(&a)) return "11: Database variables not set";
	if (32434232 != ndb_hgetEnd(&a)) return "12: Database variables not set";
	
	free(a.map);
	
	return NULL;
}

//-------------------------------------------------------------------
//Initialization/Everything else

#include "nDB/base.h"

char* base() {
	nDB a;
	nDB* ndb;
	
	//Create memory database
	nDB* db = ndb_open(NULL,0);
	
	if (!db) return "1: Failed to create database";
	
	ndb_hsetTrash(db,32343);
	
	ndb = ndb_copy(db,"testdb.db");
	
	if (!ndb) return "2: Failed to copy database";
	
	if (ndb_hgetTrash(ndb) != 32343) return "3: Copying database failed";
	
	ndb_close(db);
	ndb_close(ndb);
	
	if (ndb_open_v2(&a,"testdb.db",0)) return "4: Failed to repoen database";
	
	if (ndb_hgetTrash(&a) != 32343) return "5: Getting data from database failed";
	
	ndb_close_v2(&a);
	remove("testdb.db");
	return NULL;
}

//-------------------------------------------------------------------
//Allocator

#include "nDB/allocator.h"
char* allocator() {
	nDB* db = ndb_open("testbase.db",0);
	char* str = "Hello World!";
	NDB_LOC ptr = ndb_alloc(db,strlen(str)+1);
	
	strcpy(db->map+ptr,str);
	
	ndb_hsetTables(db,ptr);
	ptr = ndb_alloc(db,sizeof(uint64_t));
	*((uint64_t*)(db->map+ptr)) = 5463464523;
	ndb_hsetNodes(db,ptr);
	ptr = ndb_alloc(db,34564);
	
	if (ndb_hgetNodes(db) + sizeof(uint64_t) != ptr) return "1: Allocator not giving correct answer";
	
	*(db->map+ptr+34000)=54;
	
	if (ndb_hgetEnd(db) < 34564) return "2: Sizing failed";
	ndb_close(db);
	db = ndb_open("testbase.db",0);
	
	if (!(ptr = ndb_hgetTables(db))) return "3: Memory set incorrectly";
	if (strcmp(db->map+ptr,"Hello World!")) return "4: Memory incorrectly set";
	
	if (!(ptr = ndb_hgetNodes(db))) return "5: Memory set incorrectly";
	if (*((uint64_t*)(db->map+ptr)) != 5463464523) return "6: Memory failed to keep";
	
	ptr += sizeof(uint64_t);
	
	if (*(db->map+ptr+34000)!=54) return "7: Memory get failed";
	
	ndb_close(db);
	remove("testbase.db");
	return NULL;
}

//-------------------------------------------------------------------
//Misc functions

#include "nDB/misc.h"

char* misc(nDB* db) {
	uint64_t t = ndb_millis();
	//Check the boolean storage
	
	NDB_LOC l = ndb_alloc(db,1);	//Allocates a byte
	
	//Check the booleans
	if (ndb_miscGetbool(db,l,0)) return "1: Booleans not false";
	if (ndb_miscGetbool(db,l,1)) return "2: Booleans not false";
	if (ndb_miscGetbool(db,l,2)) return "3: Booleans not false";
	if (ndb_miscGetbool(db,l,3)) return "4: Booleans not false";
	if (ndb_miscGetbool(db,l,4)) return "5: Booleans not false";
	if (ndb_miscGetbool(db,l,5)) return "6: Booleans not false";
	if (ndb_miscGetbool(db,l,6)) return "7: Booleans not false";
	if (ndb_miscGetbool(db,l,7)) return "8: Booleans not false";
	
	ndb_miscSetbool(db,l,0,1);
	ndb_miscSetbool(db,l,5,0);
	ndb_miscSetbool(db,l,6,1);
	
	if (!ndb_miscGetbool(db,l,0)) return "9: Booleans incorrectly set";
	if (ndb_miscGetbool(db,l,5)) return "10: Booleans incorrectly set";
	if (!ndb_miscGetbool(db,l,6)) return "11: Booleans incorrectly set";
	if (ndb_miscGetbool(db,l,1)) return "12: Booleans not false";
	if (ndb_miscGetbool(db,l,2)) return "13: Booleans not false";
	if (ndb_miscGetbool(db,l,3)) return "14: Booleans not false";
	if (ndb_miscGetbool(db,l,4)) return "15: Booleans not false";
	if (ndb_miscGetbool(db,l,7)) return "16: Booleans not false";
	
	ndb_miscFlipbool(db,l,7);
	ndb_miscFlipbool(db,l,0);
	
	if (ndb_miscGetbool(db,l,0)) return "17: Booleans incorrectly set";
	if (ndb_miscGetbool(db,l,5)) return "18: Booleans incorrectly set";
	if (!ndb_miscGetbool(db,l,6)) return "19: Booleans incorrectly set";
	if (ndb_miscGetbool(db,l,1)) return "20: Booleans not false";
	if (ndb_miscGetbool(db,l,2)) return "21: Booleans not false";
	if (ndb_miscGetbool(db,l,3)) return "22: Booleans not false";
	if (ndb_miscGetbool(db,l,4)) return "23: Booleans not false";
	if (!ndb_miscGetbool(db,l,7)) return "24: Booleans incorrectly set";
	
	if (!t || t > ndb_millis()) return "25: Millisecond counter failed";
	
	return NULL;
}

//-------------------------------------------------------------------
//Edge manipulation
#include "nDB/edge.h"

char* edge(nDB* db) {
	NDB_LOC l = ndb_alloc(db,NDB_EDGESIZE*2);
	ndb_edgeMk(db,l,2,l+NDB_EDGESIZE,0);
	ndb_edgeSetbool(db,l,3,1);
	ndb_edgeSetdata(db,l,5335643);
	ndb_edgeMk(db,l+NDB_EDGESIZE,1,50,l);
	
	if (ndb_edgeGetfrom(db,l+NDB_EDGESIZE)!=50) return "1: Setting variables incorrect";
	if (ndb_edgeGetto(db,l+NDB_EDGESIZE)!=l) return "2: Setting variables incorrect";
	
	if (ndb_edgeGetfrom(db,l)!=l+NDB_EDGESIZE) return "3: Setting variables incorrect";
	if (ndb_edgeGetto(db,l)!=0) return "4: Setting variables incorrect";
	if (!ndb_edgeGetbool(db,l,3)) return "5: Setting variables incorrect";
	if (ndb_edgeGetdata(db,l)!=5335643) return "6: Setting variables incorrect";
	
	ndb_edgeClear(db,l);
	if (ndb_edgeGetfrom(db,l)) return "7: Clear failed";
	if (ndb_edgeGetto(db,l)) return "8: Clear failed";
	if (ndb_edgeGetbool(db,l,3)) return "9: Clear failed";
	if (ndb_edgeGetdata(db,l)) return "10: Clear failed";
	
	return NULL;
}


//-------------------------------------------------------------------
//Edge manipulation

#include "nDB/seqpg.h"

char* sequencepg(nDB* db) {
	NDB_LOC s = ndb_seqpgMk(db,0);
	
	ndb_seqpgSetprev(db,s,243234324);
	ndb_seqpgSetnext(db,s,243578654);
	ndb_seqpgSetnode(db,s,123456745);
	ndb_seqpgSetnum(db,s,987656765);
	
	if (ndb_seqpgGetprev(db,s)!=243234324) return "1: Setting variable Failed";
	if (ndb_seqpgGetnum(db,s)!=987656765) return "2: Setting variable Failed";
	if (ndb_seqpgGetnode(db,s)!=123456745) return "3: Setting variable Failed";
	if (ndb_seqpgGetnext(db,s)!=243578654) return "4: Setting variable Failed";
	return NULL;
}

//-------------------------------------------------------------------
//Node Page Manipulation

#include "nDB/node.h"

char* nodepg(nDB* db) {
	NDB_LOC n = ndb_nodeMk(db);
	
	ndb_nodeSetbool(db,n,3,1);
	ndb_nodesetData(db,n,67987654);
	ndb_nodesetSeq(db,n, 987654567);
	ndb_nodesetSeqnum(db,n, 8476278);
	ndb_nodesetLnk(db,n, 974391633);
	
	if (!ndb_nodegetbool(db,n,3)) return "1: Settings incorrect";
	if (ndb_nodegetData(db,n)!=67987654) return "2: Settings incorrect";
	if (ndb_nodegetSeq(db,n)!= 987654567) return "3: Settings incorrect";
	if (ndb_nodegetSeqnum(db,n)!= 8476278) return "4: Settings incorrect";
	if (ndb_nodegetLnk(db,n)!= 974391633) return "5: Settings incorrect";
	
	return NULL;
}

//------------------------------------------------------------------
//Sequence manipulation
#include "nDB/sequence.h"

char* sequence(nDB* db) {
	NDB_LOC e0,e1,e2,e3,e4,e5,e6,e7,e8,e9;
	ndb_iter i;
	
	NDB_LOC n = ndb_nodeMk(db);
	ndb_seqMk(db,n);
	
	e0 = ndb_seqAdd(db,n,1,2);
	e1 = ndb_seqAdd(db,n,3,4);
	e2 = ndb_seqAdd(db,n,5,4);
	e3 = ndb_seqAdd(db,n,3,6);
	e4 = ndb_seqAdd(db,n,7,4);
	e5 = ndb_seqAdd(db,n,3,8);
	e6 = ndb_seqAdd(db,n,9,4);
	e7 = ndb_seqAdd(db,n,3,10);
	e8 = ndb_seqAdd(db,n,11,4);
	e9 = ndb_seqAdd(db,n,3,12);
	
	if (!e0 || !e1 || !e2 || !e3 || !e4 || !e5 || !e6 || !e7 || !e8 || !e9)
		return "1: Adding edge failed";
	
	if (ndb_nodegetSeqnum(db,n) != 10) return "2: Incorrect numbering";
	
	if (e3 != ndb_seqGet(db,n,3)) return "3: Failed at getting";
	if (e0 != ndb_seqGet(db,n,0)) return "4: Failed at getting";
	if (e9 != ndb_seqGet(db,n,9)) return "5: Failed at getting";
	if (0 != ndb_seqGet(db,n,10)) return "6: Failed at getting";
	
	//Now we try the iterator
	ndb_seqIterinit(db,n,&i);
	
	if (ndb_seqItergetcur(&i) != e0) return "7: Failed at iterating";
	if (ndb_seqIterate(&i) != e1) return "8: Failed at iterating";
	if (ndb_seqIterate(&i) != e2) return "9: Failed at iterating";
	if (ndb_seqIterate(&i) != e3) return "10: Failed at iterating";
	if (ndb_seqIterate(&i) != e4) return "11: Failed at iterating";
	if (ndb_seqIterate(&i) != e5) return "12: Failed at iterating";
	if (ndb_seqIterate(&i) != e6) return "13: Failed at iterating";
	if (ndb_seqIterate(&i) != e7) return "14: Failed at iterating";
	if (ndb_seqIterate(&i) != e8) return "15: Failed at iterating";
	if (ndb_seqIterate(&i) != e9) return "16: Failed at iterating";
	if (ndb_seqIterate(&i) != 0) return "17: Failed at iterating";
	if (ndb_seqItergetcur(&i) != e9) return "18: Failed at iterating";
	if (ndb_seqIterateback(&i) != e8) return "19: Failed at iterating";
	if (ndb_seqIterateback(&i) != e7) return "20: Failed at iterating";
	if (ndb_seqIterateback(&i) != e6) return "21: Failed at iterating";
	if (ndb_seqIterateback(&i) != e5) return "22: Failed at iterating";
	if (ndb_seqIterateback(&i) != e4) return "23: Failed at iterating";
	if (ndb_seqIterateback(&i) != e3) return "24: Failed at iterating";
	if (ndb_seqIterateback(&i) != e2) return "25: Failed at iterating";
	if (ndb_seqIterateback(&i) != e1) return "26: Failed at iterating";
	if (ndb_seqIterateback(&i) != e0) return "27: Failed at iterating";
	if (ndb_seqIterateback(&i) != 0) return "28: Failed at iterating";
	
	if (e9 != ndb_seqIterget(&i,9)) return "29: Failed at getting";
	
	if (e2 != ndb_seqIterget(&i,2)) return "30: Failed at getting";
	
	if (e9 != ndb_seqIterget(&i,9)) return "31: Failed at getting";
	
	if (e8 != ndb_seqIterget(&i,8)) return "32: Failed at getting";
	
	
	return NULL;
}

//-------------------------------------------------------------------
//Test framework

char test(char* system, char* result) {
	if (color) {
		printf("\033[01;33mTest: %-43s\t",system);
		if (!result) printf("[\e[1;32m+\033[01;33m]\033[22;37m\n");
		else {
			haderror=1;
			printf("[\e[1;31mFAILED\033[01;33m]\e[1;31m\nERROR %s\033[22;37m\n",result);
		}
	} else {
		printf("Test: %-40s\t",system);
		if (!result) printf("[+]\n");
		else {
			haderror=1;
			printf("[FAILED]\nERROR %s\n",result);
		}

	}
	return (result?1:0);
}

int main(int argc, char** argv) {
	int lvl = 0;
	int i;
	nDB* db;
	//Parse arguments
	for (i=1;i<argc;i++) {
		if (!strcmp(argv[i],"-l") || !strcmp(argv[i],"--log")) {
			i++;
			if (i>=argc || argv[i][0]=='-') {
				printf("l/log: Logs to a file, include a file name\n");
				return 1;
			} else {
				FILE* f = fopen(argv[i],"w");
				if (f){
					ndb_logSetFile(f);
					flogg=1;
				 } else {
				 	printf("l/log: Could not open file '%s'\n",argv[i]);
				 	return 1;
				 }
			}
		} else if (!strcmp(argv[i],"-c") || !strcmp(argv[i],"--color")) {
			color = 1;
		} else if (!strcmp(argv[i],"-o") || !strcmp(argv[i],"--level")) {
			i++;
			if (i>=argc || argv[i][0]=='-') {
				printf("o/level: Sets the level at which to log at\n");
				return 1;
			} else {
				lvl = atoi(argv[i]);
			}
		} else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")) {
			printf("nDB tests - these test the nDB library for problems\n");
			printf("Options:\n");
			printf("\t-l <filename>\t: logs to the file specified, rather than terminal\n");
			printf("\t-o\t\t: Sets the log level, default is 0\n");
#if defined(linux)
			printf("\t-c\t\t: Enables color-coding terminal output\n");
#endif
			return 1;
		} else {
			printf("Command '%s' not recognized\n",argv[i]);
			return 1;
		}
	}
#if defined(__GNUC__) && defined(linux)
	mtrace();	//Yaay for mtrace!
#endif
	if (color)
		printf("\033[01;33mRunning nDB tests\033[22;37m\n");
	else
		printf("Running nDB tests\n");
		
	ndb_logSetLevel(lvl);	//Sets level so that all messages are displayed
	//Here are the tests that are carried out
	
	/************************************************************************************
	
	The tests to be carried out
	
	************************************************************************************/
	
	test("Error Handling",error());
	if (!test("File Access",filesystem())) {
		test("Header Access",header());
		if (!test("Database Core",base())) {
			if (!test("Allocator",allocator())) {
				db = ndb_open(NULL,0);
	
				test("Misc Functions",misc(db));
				test("Edge Manipulation",edge(db));
				test("Sequence Page Manipulation",sequencepg(db));
				test("Node Page Manipulation",nodepg(db));
				test("Sequence",sequence(db));
				ndb_close(db);
			}
		}
	}
	
	/************************************************************************************
	
	
	
	************************************************************************************/
	
	
	
	if (flogg)
		fclose(ndb_logGetFile());
	
#if defined(__GNUC__) && defined(linux)
	//Yay for mtrace!
	muntrace();
#endif
//Pretty color coded text - only works in linux terminal
	if (color) {
		if (!haderror) {
			printf("\n\033[01;33mAll tests completed successfully.\e[0m\n");
		} else {
			printf("\n\e[1;31mThere was a FAILURE in one or more tests.\e[0m\n");
		}
	} else {
		//In the name of portability
		if (!haderror) {
			printf("\nAll tests completed successfully.\n");
		} else {
			printf("\nThere was a FAILURE in one or more tests.\n");
		}
	}

#ifdef _WIN32
	system("pause");
#endif
	return 0;
}

//EOF

