/**************************************************************************************

                               Listing Data Index

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "listIndex.h"
#include "offsets.h"
#include "allocator.h"

NDB_LOC ndb_liGetnum(nDB* db, NDB_LOC i) {
	assert(db && i);
	if (ndb_oliID(db,i) != 'L') {
		ndb_log(NDB_LSTR,"indexIter: Location does not have identifier, database corrupted!\n");
		ndb_errSet(db,16);
		return 1;
	}
	return ndb_oliFirst(db,i);
}

NDB_LOC ndb_liMk(nDB* db,uint8_t lesize, NDB_LOC first, NDB_LOC prev) {
	NDB_LOC loc;
	assert(db && lesize);
	
	ndb_log(NDB_LSTR,"LINDEX: Creating Page\n");
	
	loc = ndb_alloc(db, NDB_LINDEXSIZE(lesize));
	if (!loc) {
		//Allocation failed - error was already set
		ndb_log(NDB_LSTR,"Lindex: Allocation failed, aborting.\n");
		return 0;
	}
	
	//Set the unit size - this has to be first, since it is used for some macros
	ndb_oliSize(db,loc) = lesize;
	
	//Check what type of page it is- if first is unset, then it is first - signified by L
	//	and if it is not the first page, signify it by using an l as the id.
	ndb_oliID(db,loc) = (first?'l':'L');
	
	//Set the previous page
	ndb_oliPrev(db,loc) = prev;
	
	//Set the first page
	ndb_oliFirst(db,loc) = first;
	
	//Aight, we set the page - return it
	return loc;
}

NDB_LOC ndb_liAdd(nDB* db, NDB_LOC index) {
	ndb_iter i;
	assert(db && index);
	if (ndb_liIterinit(db, index, &i)) return 0;	//Failed to create iterator
	return ndb_liIteradd(&i);						//The iterator-based adder
}

NDB_LOC ndb_liGet(nDB* db, NDB_LOC index,NDB_LOC num) {
	ndb_iter i;
	assert(db && index);
	if (ndb_liIterinit(db, index, &i)) return 0;	//Failed to create iterator
	return ndb_liIterget(&i,num);						//The iterator-based getter
}

//Initialize the iterator
char ndb_liIterinit(nDB* db, NDB_LOC n, ndb_iter* i) {
	assert(db && n && i);
	
	//Checks the ID of the page
	if (ndb_oliID(db,n) != 'L') {
		ndb_log(NDB_LSTR,"indexIter: Location does not have identifier, database corrupted!\n");
		ndb_errSet(db,16);
		return 1;
	}
	
	//Initialize iterator data
	i->db = db;
	i->loc = n;
	i->data = 0;
	
	return 0;
}

//Get or add
NDB_LOC ndb_liIterget(ndb_iter* i, NDB_LOC num) {
	NDB_LOC frst;
	assert(i && i->db);
	ndb_log(NDB_LSTR,"LINDEX: obtaining element %lu\n",num);
	
	//Set the first page pointer
	frst = (ndb_oliID(i->db,i->loc)=='L'?i->loc:ndb_oliFirst(i->db,i->loc));
	
	//Make sure there is at least one edge in the entire thing
	if (ndb_oliID(i->db,i->loc)=='L' && ndb_oliNum(i->db,i->loc)==0) return 0;	//There are no edges
	
	//Now, there are two options: If the number is ahead of us and if it is behind us
	//	in the iterator
	if (num > i->data) {
		NDB_LOC tot = ndb_oliFirst(i->db,frst);
		//The number is ahead of us - now we check if it would be faster going from our place or
		//from the end of the list
		if (num-i->data < tot-num) {
			//No, we want to manually move it from its location
			//First we set the iterator to the 0th place on the page
			i->data -= i->data%NDB_LINDEXPGNUM;
			while (i->data/NDB_LINDEXPGNUM < num/NDB_LINDEXPGNUM && ndb_oliNext(i->db,i->loc)) {
				//The numbers are on different pages, so we align them
				i->data += ndb_oliNum(i->db,i->loc);
				i->loc = ndb_oliNext(i->db,i->loc);
			}
		} else {
			//We reset the iterator to be at the end of the list
			//We go to the first page - if it is not there already
			if (ndb_oliID(i->db,i->loc)=='l')
				i->loc = frst;
			//Shit's gotta exist
			assert(i->loc && ndb_oliID(i->db,i->loc) == 'L');
			//The "first" is the number of items in the list
			assert(ndb_oliFirst(i->db,i->loc));
			//Sets the data to the last element (-1 because it is an index)
			i->data = ndb_oliFirst(i->db,i->loc) - 1;
			//The "prev" is the last page on the first page
			assert(ndb_oliPrev(i->db,i->loc));
			//Sets the location to the last page
			i->loc = ndb_oliPrev(i->db,i->loc);
			
			//And we leave it at that - the next if statement will pick this up and do it for us
		}
	}
	if (num < i->data) {
		//The number is behind us - check if it would be faster going here from the beginning of the list
		if (num > i->data - num) {
			//It is easier to move back from this location, so we do so.
			//First we set the iterator at the 0th place on the page
			i->data -= i->data%NDB_LINDEXPGNUM;
			while (i->data/NDB_LINDEXPGNUM > num/NDB_LINDEXPGNUM && ndb_oliPrev(i->db,i->loc)) {
				//The numbers are on different pages, so we align them
				i->loc = ndb_oliPrev(i->db,i->loc);
				i->data -= ndb_oliNum(i->db,i->loc);
			}
		} else {
			//We reset the iterator to be at the end of the list
			//There is a next page - it is not currently the last page
			//We go to the first page - if it is not there already
			if (ndb_oliID(i->db,i->loc)=='l')
				i->loc = frst;
			//Shit's gotta exist
			assert(i->loc && ndb_oliID(i->db,i->loc) == 'L');
			//The "first" is the number of items in the list
			assert(ndb_oliNext(i->db,i->loc));
			//Sets the data to the first element
			i->data = 0;
			
			//The code that moves forward is above, so we just recurse. nbd.
			return ndb_liIterget(i,num);
		}
	}
	
	//Now we are either at the right page, or it failed. We need to check for fail
	if (i->data/NDB_LINDEXPGNUM != num/NDB_LINDEXPGNUM) return 0;
	
	//We are confirmed to be at the right page - so we check if the wanted element exists
	if (num%NDB_LINDEXPGNUM >= ndb_oliNum(i->db,i->loc))
		return 0;	//The element does not exist
	
	//Set the data
	i->data = num;
	return ndb_oliGet(i->db,i->loc,num%NDB_LINDEXPGNUM);
}

NDB_LOC ndb_liIteradd(ndb_iter* i) {
	assert(i && i->db);
	
	ndb_log(NDB_LSTR,"LINDEX: adding element\n");
	
	//i was already checked at iterator initialization
	//First we check if it is the last page
	if (ndb_oliNext(i->db,i->loc)) {
		//There is a next page - it is not currently the last page
		//We go to the first page - if it is not there already
		if (ndb_oliID(i->db,i->loc)=='l')
			i->loc = ndb_oliFirst(i->db,i->loc);
		//Shit's gotta exist
		assert(i->loc && ndb_oliID(i->db,i->loc) == 'L');
		//The "first" is the number of items in the list
		assert(ndb_oliFirst(i->db,i->loc));
		//Sets the data to the last element (-1 because it is an index)
		i->data = ndb_oliFirst(i->db,i->loc) - 1;
		//The "prev" is the last page on the first page
		assert(ndb_oliPrev(i->db,i->loc));
		//Sets the location to the last page
		i->loc = ndb_oliPrev(i->db,i->loc);
	} else {
		//We are on the last page - move us to the last element
		if (ndb_oliNum(i->db,i->loc))
			i->data += ndb_oliNum(i->db,i->loc) - ((i->data % NDB_LINDEXPGNUM)+1);
	}
	
	//Assert we are at the right place
	assert(!ndb_oliNum(i->db,i->loc) || i->data % NDB_LINDEXPGNUM+1 == ndb_oliNum(i->db,i->loc));
	
	//If we are supposed to make a new page, notify about it right about now
	//	If it is the first element, as in there are no elements on the page yet, do not iterate
	//	This is to combat the first-value failure
	if (ndb_oliNum(i->db,i->loc) && ((++i->data) % NDB_LINDEXPGNUM)==0) {
		//Yes, please DO make a new page
		NDB_LOC l;
		if (ndb_oliID(i->db,i->loc) != 'L')
			l = ndb_liMk(i->db,ndb_oliSize(i->db,i->loc),ndb_oliFirst(i->db,i->loc),i->loc);
		else
			l = ndb_liMk(i->db,ndb_oliSize(i->db,i->loc),i->loc,i->loc);
		
		if (!l) {
			i->data--;
			ndb_log(NDB_LSTR,"Lindex: Failed to allocate new page\n");
			return 0;
		}
		
		//Set the "next page"
		ndb_oliNext(i->db,i->loc) = l;
		
		//Set the "last page" link on the first page
		if (ndb_oliID(i->db,i->loc)=='l')
			ndb_oliPrev(i->db,ndb_oliFirst(i->db,i->loc)) = l;
		else
			ndb_oliPrev(i->db,i->loc) = l;
		//Switch the location to the new page
		i->loc = l;
	}
	
	//We are on the right page, and data has been iterated to the new location
	ndb_oliNum(i->db,i->loc)++;
	
	//Set the "last page" on the first page, so the number of elements in the list
	if (ndb_oliID(i->db,i->loc)=='l')
		ndb_oliFirst(i->db,ndb_oliFirst(i->db,i->loc)) = i->data+1;
	else
		ndb_oliFirst(i->db,i->loc) = i->data+1;
		
	//Give the element
	return ndb_oliGet(i->db,i->loc,i->data%NDB_LINDEXPGNUM);
}

//Iterate forward and back on the index
NDB_LOC ndb_liIterate(ndb_iter* i) {
	NDB_LOC res;
	assert(i);
	
	//If it is about to go over the page limits - so if it is supposed to switch pages
	if (i->data !=0 && i->data % NDB_LINDEXPGNUM == 0) {
		if (ndb_oliNext(i->db,i->loc)) {
			//There IS a next page
			i->loc = ndb_oliNext(i->db,i->loc);
		} else return 0;	//This means that the iteration got to the end
	}
	//The element does not exist
	if (i->data%NDB_LINDEXPGNUM >= ndb_oliNum(i->db,i->loc)) return 0;

	res = ndb_oliGet(i->db,i->loc,(i->data++)%NDB_LINDEXPGNUM);
	
	return res;
}
NDB_LOC ndb_liIterateback(ndb_iter* i) {
	assert(i);
	
	if (i->data % NDB_LINDEXPGNUM == 0) {
		if (ndb_oliPrev(i->db,i->loc) && i->data != 0) {
			//There IS a previous page
			i->loc = ndb_oliPrev(i->db,i->loc);
		} else return 0;
	}
	return ndb_oliGet(i->db,i->loc,(--i->data)%NDB_LINDEXPGNUM);
}

NDB_LOC ndb_liIteratecur(ndb_iter* i) {
	assert(i && i->db);
	if (i->data ==0)
		return ndb_oliGet(i->db,i->loc,i->data%NDB_LINDEXPGNUM);
	else
		return ndb_oliGet(i->db,i->loc,(i->data-1)%NDB_LINDEXPGNUM);
}

//EOF

