/**************************************************************************************

                                         Cache
                                       Main Header

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_CACHE
#	define NDB_CACHE

#include "ndb_def.h"

/*
	The cache is the part of the database where all of the data is stored during runtime.
	All pages that are modified stay in the cache, so that there is fast data-access.
*/

char ndb_cacheInit(nDB* db);
void ndb_cacheClose(nDB* db);
	//These two functions initialize and free the cache. The first returns a boolean, 1=True,
	//	0=false, signifying if it was successful at initializing.
	
ndb_page* ndb_cacheGet(nDB* db, NDB_PGNUM page);
	//Gets the page from cache, returns NULL if the page is not in cache

ndb_page* ndb_cacheNewPage(nDB* db);
void ndb_cacheDelPage(nDB* db, ndb_page* p);
	//These make and get back cache pages. This uses a page pool to optimally manage memory

char ndb_cacheAddPage(nDB* db, ndb_page* p);
void ndb_cacheRemovePage(nDB* db, ndb_page* p);
	//Adds and removes pages from the cache - remove frees the page automatically

void ndb_cacheClean(nDB* db);
	//Deletes any unused pages to free as much memory as possible

typedef uint64_t ndb_cacheIter;	
	//The cache iterator allows for quick iterating through the pages in the cache

ndb_page* ndb_cacheIterate(nDB* db,ndb_cacheIter* i);
	//It iterates through the cache one at a time

//-------------------------------------------------------------------------------------
//Page-specific functions - these allow for the database to be multithreaded
//WARNING: I have assumed there are at most 255 readers at a time - so it will fail
//	once it goes over that number

uint8_t ndb_pgInUse(ndb_page* p);		//Checks if the page is in use at all
uint8_t ndb_pgReaders(ndb_page* p);		//Returns the current amount of readers
uint8_t ndb_pgIsLocked(ndb_page* p);	//Is the page locked?
uint8_t ndb_pgIsModified(ndb_page* p);	//Is the page modified?

void ndb_pgRead_(ndb_page* p);			//Add a reader to the page
void ndb_pgDone_(ndb_page* p);			//Remove a reader from the page

void ndb_pgSetWriteLock_(ndb_page* p);	//Sets the write lock, and the modified flag!
void ndb_pgWriteLock_(ndb_page* p);		//Set write lock and busywait until no more readers
void ndb_pgWriteUnlock_(ndb_page* p);	//Unlock from write

#define ndb_pgRead ndb_pgRead_
#define ndb_pgDone ndb_pgDone_
#define ndb_pgSetWriteLock ndb_pgSetWriteLock_
#define ndb_pgWriteLock ndb_pgWriteLock_
#define ndb_pgWriteUnlock  ndb_pgWriteUnlock_

#if defined(NDB_PLVER) && NDB_HASLOG
#undef ndb_pgRead
#undef ndb_pgDone
#undef ndb_pgSetWriteLock
#undef ndb_pgWriteLock
#undef ndb_pgWriteUnlock

#define ndb_pgRead ndb_pgRead_l
#define ndb_pgDone ndb_pgDone_l
#define ndb_pgSetWriteLock ndb_pgSetWriteLock_l
#define ndb_pgWriteLock ndb_pgWriteLock_l
#define ndb_pgWriteUnlock  ndb_pgWriteUnlock_l

#define ndb_pgRead_l(a) ndb_pgRead__(a,__FILE__,__LINE__)
#define ndb_pgDone_l(a) ndb_pgDone__(a,__FILE__,__LINE__)
#define ndb_pgSetWriteLock_l(a) ndb_pgSetWriteLock__(a,__FILE__,__LINE__)
#define ndb_pgWriteLock_l(a) ndb_pgWriteLock__(a,__FILE__,__LINE__)
#define ndb_pgWriteUnlock_l(a)  ndb_pgWriteUnlock__(a,__FILE__,__LINE__)

void ndb_pgRead__(ndb_page* p,char* fname,int lnum);
void ndb_pgDone__(ndb_page* p,char* fname,int lnum);
void ndb_pgSetWriteLock__(ndb_page* p,char* fname,int lnum);
void ndb_pgWriteLock__(ndb_page* p,char* fname,int lnum);
void ndb_pgWriteUnlock__(ndb_page* p,char* fname,int lnum);

#endif

#endif
//EOF

