/**************************************************************************************

                                    Error Handling
                                    Implementation

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "error.h"

/*
Currently, it is as simple as it gets, a basic error struct embedded in the nDB object
TODO: Make it multithread ready, so each thread has its own error-pipeline.
*/

//-------------------------------------------------------------------------------------
//Error String Definitions
#define ESLEN 6
char* errstring[ESLEN] = {	//The database of the error strings
	"Hakuna Matata",	//0: No errors
	"The database is corrupted",	//1: Corrupted
	"Expected an object, got NULL",	//2: NULL
	"Not enough memory",			//3: Memory
	"Unknown Error",				//4: Unknown
	"Maximum cache size reached"	//5: Cache
};

//-------------------------------------------------------------------------------------
//Function Implementation

void ndb_errinit(nDB* db) {
	//Here, we simply set the error number in the database to 0. No biggie.
	//	We assume that it is called by nDB only
	assert(db != NULL);
	db->err =0;
}

void ndb_errclose(nDB* db) {
	//Same to the previous command, this time it closes the database. Currently, nothing happens
	assert(db != NULL);
	//This added to catch any bugs, since this might be used in the future
}

uint8_t ndb_peekerr(nDB* db) {
	if (!db)	//This allows for checking error in creation.
		return NDBERR_NULL;
		
	assert(db->err < ESLEN);
		//The error can't be out of range at this point, since the value was 
		//	already checked!
	
	return db->err;	//Not that hard, was it now?
	
	//FUN FACT: This always returns something valid.
}

uint8_t ndb_geterr(nDB* db) {
	uint8_t num = ndb_peekerr(db);
	db->err=0;	//Resets the error
	return num;
}

char* ndb_geterrStr(nDB* db) {
	return errstring[ndb_geterr(db)];	//The strings are in error order
}

char* ndb_geterrString(uint8_t e) {
	return errstring[e];
}
void ndb_seterr_(nDB* db, uint8_t e) {
	//Once again, only nDB should be calling this, and it already made sure that db not NULL
	assert(db != NULL);
	assert (e < ESLEN); //same reasoning as above, it should be calling errors known to it!
	
	db->err = e;	//Simple as that
}

#if defined(NDB_EVER) && NDB_HASLOG	//For logging
void ndbver_seterr(nDB* db, uint8_t e,char* loc,int line) {
	ndb_log(NDB_EVER,"\n(NDBERR %s | %s:%d )",ndb_geterrString(e),loc,line);
	ndb_seterr_(db,e);
}
#endif

//EOF

