/**************************************************************************************

                                  nDB Graph Edge

**************************************************************************************/
//Tab Width: 4 Character:\t

#ifndef NDB_EDGE
#	define NDB_EDGE


#include "ndb_def.h"
#include "misc.h"		//Allows simple boolean stuffs

/*
Functions to manipulate edges. An edge's structure is as follows:

uint8_t,uint8_t,NDB_LOC,NDB_LOC,NDB_LOC
uint8_t -> bools, 8 of them
uint8_t -> the number this link is on the sequencePage.
	This automatically limits the linknumber in a sequence page to 256!
LOC_1 -> from link
LOC_2 -> to link
LOC_3 -> Datalink/data (first boolean in uint distinguishes if this is a link or data

*/

//The size of a link
#define NDB_EDGESIZE (2*sizeof(uint8_t)+3*sizeof(NDB_LOC))

void ndb_edgeMk_(nDB* db, NDB_LOC l, uint8_t num, NDB_LOC from, NDB_LOC to);

void ndb_edgeClear_(nDB* db, NDB_LOC e);					//Sets the edge to all-NULL

//Since the boolean is the first thing in each link, it can just redefine the boolfuncs
#define ndb_edgeGetbool ndb_miscGetbool
#define ndb_edgeSetbool ndb_miscSetbool
#define ndb_edgeFlipbool ndb_miscFlipbool

//Get and set edge data
uint8_t ndb_edgeGetsnum(nDB* db, NDB_LOC l);
NDB_LOC ndb_edgeGetfrom(nDB* db, NDB_LOC e);			//
NDB_LOC ndb_edgeGetto(nDB* db, NDB_LOC e);
NDB_LOC ndb_edgeGetdata(nDB* db, NDB_LOC e);

void ndb_edgeSetsnum_(nDB* db, NDB_LOC l, uint8_t num);
void ndb_edgeSetfrom_(nDB* db, NDB_LOC e,NDB_LOC from);
void ndb_edgeSetto_(nDB* db, NDB_LOC e, NDB_LOC to);
void ndb_edgeSetdata_(nDB* db, NDB_LOC e, NDB_LOC data);

#if NDB_LOG
#define ndb_edgeMk(a,b,c,d,e) ndb_edgeMk__(a,b,c,d,e,NDB_LD)
#define ndb_edgeClear(a,b) ndb_edgeClear__(a,b,NDB_LD)
#define ndb_edgeSetsnum(a,b,c) ndb_edgeSetsnum__(a,b,c,NDB_LD)
#define ndb_edgeSetfrom(a,b,c) ndb_edgeSetfrom__(a,b,c,NDB_LD)
#define ndb_edgeSetto(a,b,c) ndb_edgeSetto__(a,b,c,NDB_LD)
#define ndb_edgeSetdata(a,b,c) ndb_edgeSetdata__(a,b,c,NDB_LD)

void ndb_edgeMk__(nDB* db,NDB_LOC e, uint8_t num,NDB_LOC from, NDB_LOC to,NDB_LF);
void ndb_edgeClear__(nDB* db, NDB_LOC e,NDB_LF);
void ndb_edgeSetsnum__(nDB* db, NDB_LOC e, uint8_t num,NDB_LF);
void ndb_edgeSetfrom__(nDB* db, NDB_LOC e,NDB_LOC from,NDB_LF);
void ndb_edgeSetto__(nDB* db, NDB_LOC e, NDB_LOC to,NDB_LF);
void ndb_edgeSetdata__(nDB* db, NDB_LOC e, NDB_LOC data,NDB_LF);

#else
#define ndb_edgeMk ndb_edgeMk_
#define ndb_edgeClear ndb_edgeClear_
#define ndb_edgeSetsnum ndb_edgeSetsnum_
#define ndb_edgeSetfrom ndb_edgeSetfrom_
#define ndb_edgeSetto ndb_edgeSetto_
#define ndb_edgeSetdata ndb_edgeSetdata_
#endif

#endif
//EOF

