/**************************************************************************************

                                 Database Header
                                    Definitions

**************************************************************************************/
//Tab Width: 4 Character:\t

#include "header.h"

#include <string.h>	//Memset plox
#include <stdio.h>	//Simple file access functions

//-------------------------------------------------------------------------------------

//Checks if a file is a database
char ndb_checkFile_(char* filename) {
	char id[5];		//Temporary variables
	uint8_t i=0;
	FILE* f;
	assert(filename);	//There has to be a filename passed
	
	//Try opening the file
	if ((f = fopen(filename,"r"))) {
		memset(id,'\0',5);	//Clear the string
		
		fread(id,4,1,f);
		
		if (!strcmp(id,"nDB")) {
			i = getc(f);
			if (i <= NDB_VER) {
				fclose(f);
				return 0;	//Looks good
			} else {
				fclose(f);
				ndb_log(NDB_LMSG,"fcheck: Database version mismatch");
				return 1;	//There is a version mismatch, so it can't read
			}
			
		}
		ndb_log(NDB_LMSG,"fcheck: File is not an nDB database");
		fclose(f);
		return 2;
	}
	
	ndb_log(NDB_LMSG,"fcheck: File does not exist yet");
	return -1;	//A failure, not a database file or can't open
}

//Writes the default header
void ndb_hwrite_(nDB* db) {
	assert(db && db->map);
	assert(db->size >= 5+4*sizeof(NDB_LOC));	//The map size has to be large enough to hold the header
	
	strcpy(db->map,"nDB");						//Write the starting bytes of the database
	db->map[4] = NDB_VER;						//Write the database version
	
	ndb_hsetTrash(db,0);						//There is no trash-linked list
	ndb_hsetTables(db,0);						//The location of the table structure
	ndb_hsetNodes(db,0);						//The location of the root linkpage
	
	ndb_hsetEnd(db,5+4*sizeof(NDB_LOC));		//Write the size at which the database ends
	
}

//Gets and sets the database ending location
uint64_t ndb_hgetEnd(nDB* db) {	
	assert(db && db->map);
	return (*((NDB_LOC*)(db->map+5)));
}
void ndb_hsetEnd_(nDB* db, uint64_t end) {
	assert(db && db->map);
	(*((NDB_LOC*)(db->map+5))) = end;
}

//Gets and sets the tables structure
NDB_LOC ndb_hgetTables(nDB* db) {
	assert(db && db->map);
	return (*((NDB_LOC*)(db->map+5+3*sizeof(NDB_LOC))));
}
void ndb_hsetTables_(nDB* db, NDB_LOC end) {
	assert(db && db->map);
	(*((NDB_LOC*)(db->map+5+3*sizeof(NDB_LOC)))) = end;
}

NDB_LOC ndb_hgetNodes(nDB* db) {
	assert(db && db->map);
	return (*((NDB_LOC*)(db->map+5+2*sizeof(NDB_LOC))));
}
void ndb_hsetNodes_(nDB* db, NDB_LOC end) {
	assert(db && db->map);
	(*((NDB_LOC*)(db->map+5+2*sizeof(NDB_LOC)))) = end;
}

//Gets and sets the trash list's pointer
NDB_LOC ndb_hgetTrash(nDB* db) {
	assert(db && db->map);
	return (*((NDB_LOC*)(db->map+5+sizeof(NDB_LOC))));
}
void ndb_hsetTrash_(nDB* db, NDB_LOC end) {
	assert(db && db->map);
	(*(NDB_LOC*)(db->map+5+sizeof(NDB_LOC))) = end;
}

#if NDB_LOG

char ndb_checkFile__(char* filename,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| FCHECK: Checking if '%s' is an nDB file\n",f,l,filename);
	return ndb_checkFile_(filename);
}
void ndb_hwrite__(nDB* db,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| HWRITE: Writing new database header\n",f,l);
	ndb_hwrite_(db);
}
void ndb_hsetEnd__(nDB* db, NDB_LOC end,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| SETEND: Setting new database endpoint %lu->%lu\n",f,l,ndb_hgetEnd(db),end);
	ndb_hsetEnd_(db,end);
}
void ndb_hsetTables__(nDB* db, NDB_LOC end,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| SETTBL: Setting new tables link %lu->%lu\n",f,l,ndb_hgetTables(db),end);
	ndb_hsetTables_(db,end);
}
void ndb_hsetNodes__(nDB* db, NDB_LOC end,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| SETNDE: Setting new Node link %lu->%lu\n",f,l,ndb_hgetNodes(db),end);
	ndb_hsetNodes_(db,end);
}
void ndb_hsetTrash__(nDB* db, NDB_LOC end,NDB_LF) {
	ndb_ilog(NDB_LHDR,"%8.8s:%-4d| SETTRS: Setting new trashroot %lu->%lu\n",f,l,ndb_hgetTrash(db),end);
	ndb_hsetTrash_(db,end);
}

#endif

//EOF

