%{
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <math.h>
#include <string.h>

int yylex();
int yyparse();

void yyerror(char* s);
void unquote(char* s);
//FILE* yyin;
%}
%error-verbose
//%pure-parser
%union {
	int ival;
	double fval;
	char* sval;
}

//Define the Tokens
%token SELECT SET GET WHERE
%token NODE LINK TABLE
%token AND OR NEQ LEQ GEQ DEQ
%token <ival> INT
%token <fval> FLOAT
%token <sval> STRING ID

%type <fval> fexp
%type <ival> iexp
%type <sval> sexp

%left '+' '-'
%left '*' '/'
%left NEG
%right '^'


%%


query:
	SELECT iexp { printf("Result (Int):%i\n",$2); }
	| SELECT fexp { printf("Result (Float):%f\n",$2);}
	| SELECT sexp { printf("Result (String):%s\n",$2);}
	;

iexp:
	INT					{ $$ = $1; }
	| iexp '+' iexp		{ $$ = $1 + $3 ;}
	| iexp '-' iexp		{ $$ = $1 - $3 ;}
	| '(' iexp ')'		{ $$ = $2 }
	| iexp '*' iexp		{ $$ = $1 * $3 ;}
	| iexp '/' iexp		{ $$ = $1 / $3 ;}
	| iexp '^' iexp		{ $$ = (int)pow((double)$1,(double)$3);}
	| '-' iexp %prec NEG { $$ = -$2 }
	;
fexp:
	FLOAT				{ $$ = $1; }
	| '(' fexp ')'		{ $$ = $2; }
	| iexp '*' fexp		{ $$ = ((double)$1) * $3 ;}
	| iexp '/' fexp		{ $$ = ((double)$1) / $3 ;}
	| fexp '*' iexp		{ $$ = $1 * $3 ;}
	| fexp '/' iexp		{ $$ = $1 / $3 ;}
	| fexp '+' iexp		{ $$ = $1 + $3 ;}
	| fexp '-' iexp		{ $$ = $1 - $3 ;}
	| fexp '^' fexp		{ $$ = pow($1,$3);}
	| iexp '+' fexp		{ $$ = $1 + $3 ;}
	| iexp '-' fexp		{ $$ = $1 - $3 ;}
	| fexp '+' fexp		{ $$ = $1 + $3 ;}
	| fexp '-' fexp		{ $$ = $1 - $3 ;}
	| '-' fexp %prec NEG { $$ = -$2; }
	;
sexp:
	STRING				{	
							//This is the most difficult part of string manipulation,
							//	Since strings have quotes around them, and they allow
							//	backslash delimited stuff, this has to parse that and 
							//	get rid of the garbage.
							unquote($1);
							$$ = $1;
						}
	| sexp '+' sexp		{
							char* str = malloc(strlen($1)+strlen($3)+1);
							strcpy(str,$1);
							strcat(str,$3);
							free($1);	//Free the two old strings
							free($3);
							$$ = str;
						}
	;
%%
void unquote(char* val) {
	int i;
	int curloc = 0;
	int slen = strlen(val);
	
	for (i=1; i < slen-1; i++)
		if (val[i]=='\\') {
			switch (val[i+1]) {
				case '\\':
					val[curloc++]='\\';
					i++;
					break;
				case '\'':
					val[curloc++]='\'';
					i++;
					break;
				case '\"':
					val[curloc++]='\"';
					i++;
					break;
				default:
					val[curloc++]=val[i];
					break;
			}
		}
		else {
			val[curloc++]=val[i];
		}
	val[curloc]='\0';
							
}
void yyerror(char* s) {
	printf("Error: %s\n",s);
}

